from silmused.ChecksLayer import ChecksLayer
from silmused.tests.QueryStructureTest import QueryStructureTest
from silmused.tests.QueryDataTest import QueryDataTest

tests = [
    ChecksLayer(
        title='Ülesanne 2 Päringu kontrollid',
        tests=[
            QueryStructureTest(
                title='Kas on olemas veerg lavanimi?',
                name='query_test',
                column_name='lavanimi',
                points=5,
            ),
            QueryStructureTest(
                title='Kas on olemas veerg sugu?',
                name='query_test',
                column_name='sugu',
                points=5,
            ),
            QueryStructureTest(
                title='Kas on olemas veerg pealkiri?',
                name='query_test',
                column_name='pealkiri',
                points=5,
            ),
            QueryStructureTest(
                title='Kas on olemas veerg punktid_finaalis?',
                name='query_test',
                column_name='punktid_finaalis',
                points=5,
            ),
            QueryStructureTest(
                title='Kas on olemas veerg koht_finaalis?',
                name='query_test',
                column_name='koht_finaalis',
                points=5,
            ),
            QueryDataTest(
                title='Kas on õige ridade arvuga tulemus?',
                name='query_test',
                column_name='COUNT(*)',
                expected_value=5,
                points=15,
            ),
            QueryDataTest(
                title='Kas nimekirjas on ainult finalistid?',
                name='query_test',
                column_name='koht_finaalis',
                expected_value="None",
                should_exist=False,
                points=30,
            ),
            QueryDataTest(
                title='Kas on õige järjekord?',
                name='query_test',
                column_name='lavanimi',
                where="test_id=2",
                expected_value="Wrs",
                points=30,
            ),
        ]
    ),
]


from silmused.ChecksLayer import ChecksLayer
from silmused.ExecuteLayer import ExecuteLayer
from silmused.tests.DataTest import DataTest
from silmused.tests.StructureTest import StructureTest
from silmused.tests.ConstraintTest import ConstraintTest
from silmused.tests.FunctionTest import FunctionTest
from silmused.tests.IndexTest import IndexTest
from silmused.tests.ProcedureTest import ProcedureTest
from silmused.tests.TriggerTest import TriggerTest
from silmused.tests.ViewTest import ViewTest

_user1 = 123456
_user2 = 123457
_partii_id = 123123
_asula_id = 200
_asula_nimi = 'Test_asula'
_klubi_id = 201
_klubi_nimi = 'Klubi kustutamiseks'

tests = [
    ChecksLayer(
        title='Single test',
        tests=[
            # 8
            DataTest(
                title='Kas isiku Irys perenimi on muudetud?',
                name='isikud',
                column_name='perenimi',
                where="eesnimi = 'Irys'",
                expected_value='Kompvek1',
                custom_feedback="test test",
                points=3,
            ),
        ],
    ),
]


from silmused.ChecksLayer import ChecksLayer
from silmused.tests.QueryStructureTest import QueryStructureTest
from silmused.tests.QueryDataTest import QueryDataTest
tests = [
    ChecksLayer(
        title='Ülesanne 7 Päringu kontrollid',
        tests=[
            QueryStructureTest(
                title='Kas on olemas veerg nimi?',
                name='query_test',
                column_name="nimi",
                points=20,
            ),
            QueryStructureTest(
                title='Kas on olemas veerg klubide arv?',
                name='query_test',
                column_name="klubide arv",
                points=20,
            ),
            QueryDataTest(
                title='Kas on õige ridade arvuga tulemus?',
                name='query_test',
                column_name='COUNT(*)',
                expected_value=10,
                points=20,
            ),
            QueryDataTest(
                title='Kas on õige linn kõige suurema klubide arvuga?',
                name='query_test',
                column_name='"klubide arv"',
                where="nimi = 'Tartu'",
                expected_value=6,
                points=30,
                column_name_fallback=['klubide%arv'],
            ),
            QueryDataTest(
                title='Kas tulemuses on näha klubideta asulad?',
                name='query_test',
                column_name='"klubide arv"',
                where="nimi = 'Elva'",
                expected_value=0,
                points=30,
                column_name_fallback=['klubide%arv'],
            ),
        ]
    ),
]


from silmused.TitleLayer import TitleLayer
from silmused.ChecksLayer import ChecksLayer
from silmused.ExecuteLayer import ExecuteLayer
from silmused.tests.DataTest import DataTest
from silmused.tests.StructureTest import StructureTest
from silmused.tests.ConstraintTest import ConstraintTest
from silmused.tests.FunctionTest import FunctionTest
from silmused.tests.IndexTest import IndexTest
from silmused.tests.ProcedureTest import ProcedureTest
from silmused.tests.TriggerTest import TriggerTest
from silmused.tests.ViewTest import ViewTest
"""
tests = [
    # Kodutöö 3

    # Ülesanne 1 (30p) ja 2 (10p)
    ChecksLayer(
        title='Tabeli Inimesed kontrollid',
        tests=[
            StructureTest(
                title='Kas tabel Inimesed on olemas?',
                name='inimesed',
                points=6,
            ),
            StructureTest(
                title='Kas veerg eesnimi on olemas?',
                name='inimesed',
                column_name='eesnimi',
                points=3,
            ),
            StructureTest(
                title='Kas veerg perenimi on olemas?',
                name='inimesed',
                column_name='perenimi',
                points=3,
            ),
            StructureTest(
                title='Kas veerg sugu on olemas?',
                name='inimesed',
                column_name='sugu',
                points=3,
            ),
            ConstraintTest(
                title='Kas veerul sugu on check kitsendus?',
                name='inimesed',
                #column_name='sugu',
                constraint_type='CHECK',
                points=3,
            ),
            StructureTest(
                title='Kas veerg synnipaev on olemas?',
                name='inimesed',
                column_name='synnipaev',
                points=3,
            ),
            StructureTest(
                title='Kas veerg sisestatud on olemas?',
                name='inimesed',
                column_name='sisestatud',
                points=3,
            ),
            StructureTest(
                title='Kas veerg isikukood on olemas?',
                name='inimesed',
                column_name='isikukood',
                points=3,
            ),
            ConstraintTest(
                title='Kas veerul isikukood on unikaalsuse kitsendus?',
                name='inimesed',
                column_name='isikukood',
                constraint_type='UNIQUE',
                points=3,
            ),
            DataTest(
                title='Kas tabelis inimesed on andmed olemas?',
            name='inimesed',
            points=10,
            ),
        ]
    ),
    # Ülesande 3 (10p), 4 (20p), 5 (20p), 6 (10p) kontroll
    ChecksLayer(
        title='Tabeli Turniirid kontrollid',
        tests=[
            # Ülesande 3
            StructureTest(
                title='Kas veerg asula on olemas?',
                name='turniirid',
                column_name='asula',
                points=10,
            ),

            # Ülesande 4
            DataTest(
                title='Kas veergu asula on väärtused sisestatud?',
                name='turniirid',
                column_name='asula',
                points=20,
            ),
            # Ülesande 5
            ConstraintTest(
                title='Kas tabelis on välisvõti olemas?',
                name='turniirid',
                constraint_type='FOREIGN KEY',
                points=20,
            ),
            # Ülesande 6 
            StructureTest(
                title='Kas veerg toimumiskoht on kustutatud?',
                name='turniirid',
                column_name=['toimumiskoht','asukoht'],
                should_exist=False,
                points=10,
            ),
        ]
    ),
]
"""
"""
tests = [
    # Kodutöö 4 kontrollid
    ChecksLayer(
        title='Vaate v_turniiripartiid kontrollid',
        tests=[
            ViewTest(
                title='Kas vaade v_turniiripartiid on olemas?',
                name='v_turniiripartiid',
                points=1,
            ),
            ViewTest(
                title='Kas veerg turniir_nimi on olemas?',
                name='v_turniiripartiid',
                column_name='turniir_nimi',
                points=1,
            ),
            ViewTest(
                title='Kas veerg toimumiskoht on olemas?',
                name='v_turniiripartiid',
                column_name='toimumiskoht',
                points=1,
            ),
            ViewTest(
                title='Kas veerg partii_id on olemas?',
                name='v_turniiripartiid',
                column_name='partii_id',
                points=1,
            ),
            ViewTest(
                title='Kas veerg partii_algus on olemas?',
                name='v_turniiripartiid',
                column_name='partii_algus',
                points=1,
            ),
            ViewTest(
                title='Kas veerg partii_lopp on olemas?',
                name='v_turniiripartiid',
                column_name='partii_lopp',
                points=1,
            ),
            ViewTest(
                title='Kas veerg kes_voitis on olemas?',
                name='v_turniiripartiid',
                column_name='kes_voitis',
            ),
            DataTest(
                title='Kas vaate andmete kogus on õige?',
                name='v_turniiripartiid',
                column_name='COUNT(*)',
                expected_value=299,
                isView=True,
                points=1,
            ),
            DataTest(
                title='Kas partii 270 võitja tulemus on õige?',
                name='v_turniiripartiid',
                column_name='LOWER(kes_voitis)',
                where='partii_id = 270',
                expected_value='valge',
                isView=True,
                points=1,
            ),
            DataTest(
                title='Kas partii 241 võitja tulemus on õige?',
                name='v_turniiripartiid',
                column_name='LOWER(kes_voitis)',
                where='partii_id = 241',
                expected_value='must',
                isView=True,
                points=1,
            ),
            DataTest(
                title='Kas partii 193 sai viigi?',
                name='v_turniiripartiid',
                column_name='LOWER(kes_voitis)',
                where='partii_id = 193',
                expected_value='viik',
                isView=True,
                points=1,
            ),

        ]
    ),
    ChecksLayer(
        title='Vaate v_klubipartiikogused kontrollid',
        tests=[
            ViewTest(
                title='Kas vaade v_klubipartiikogused on olemas?',
                name='v_klubipartiikogused',
                points=1,
            ),
            ViewTest(
                title='Kas veerg klubi_nimi on olemas?',
                name='v_klubipartiikogused',
                column_name='klubi_nimi',
                points=1,
            ),
            ViewTest(
                title='Kas veerg partiisid on olemas?',
                name='v_klubipartiikogused',
                column_name='partiisid',
                points=1,
            ),
            DataTest(
                title='Kas vaate andmete kogus on õige?',
                name='v_klubipartiikogused',
                column_name='COUNT(*)',
                expected_value=12,
                isView=True,
                points=1,
            ),
            DataTest(
                title='Kas vaate partiide summa kokku on õige?',
                name='v_klubipartiikogused',
                column_name='SUM(partiisid)',
                expected_value=571,
                isView=True,
                points=1,
            ),
        ]
    ),

    ChecksLayer(
        title='Vaate v_keskminepartii kontrollid',
        tests=[
            ViewTest(
                title='Kas vaade v_keskminepartii on olemas?',
                name='v_keskminepartii',
                points=1,
            ),
            ViewTest(
                title='Kas veerg turniiri_nimi on olemas?',
                name='v_keskminepartii',
                column_name='turniiri_nimi',
                points=1,
            ),
            ViewTest(
                title='Kas veerg keskmine_partii on olemas?',
                name='v_keskminepartii',
                column_name='keskmine_partii',
                points=1,
            ),
            DataTest(
                title='Kas vaate andmete kogus on õige?',
                name='v_keskminepartii',
                column_name='COUNT(*)',
                expected_value=5,
                isView=True,
                points=1,
            ),
            DataTest(
                title='Kas turniiril Plekkkarikas 2010 keskmine partii arv on õige?',
                name='v_keskminepartii',
                column_name='ROUND(keskmine_partii, 3)',
                where="turniiri_nimi = 'Plekkkarikas 2010'",
                expected_value=23.765,
                isView=True,
                points=1,
            ),
            DataTest(
                title='Kas turniiril Kolme klubi kohtumine keskmine partii arv on õige?',
                name='v_keskminepartii',
                column_name='ROUND(keskmine_partii, 3)',
                where="turniiri_nimi = 'Kolme klubi kohtumine'",
                expected_value=23.040,
                isView=True,
                points=1,
            ),
        ]
    ),
    ChecksLayer(
        title='Materialiseeritud vaate mv_partiide_arv_valgetega kontrollid',
        tests=[
            ViewTest(
                title='Kas vaade mv_partiide_arv_valgetega on olemas?',
                name='mv_partiide_arv_valgetega',
                isMaterialized=True,
                points=1,
            ),
            DataTest(
                title='Kas vaate andmete kogus on õige?',
                name='mv_partiide_arv_valgetega',
                column_name='COUNT(*)',
                expected_value=85,
                isView=True,
                points=1,
            ),
            DataTest(
                title='Kas leidub partii ühe mängijaga?',
                name='mv_partiide_arv_valgetega',
                column_name='COUNT(*)',
                where="eesnimi = 'Tarmo' AND perenimi = 'Kooser'",
                isView=True,
                points=1,
            ),
            DataTest(
                title='Kas vaate kõige väikseim väärtus on õige?',
                name='mv_partiide_arv_valgetega',
                column_name='MIN(partiisid_valgetega)',
                expected_value=0,
                isView=True,
                points=1,
            ),
            DataTest(
                title='Kas vaate kõige suurem väärtus on õige?',
                name='mv_partiide_arv_valgetega',
                column_name='MAX(partiisid_valgetega)',
                expected_value=14,
                isView=True,
                points=1,
            )
        ]
    ),
]
"""
"""
tests = [
    # Kodutöö 5 kontrollid
    # 25p
    ChecksLayer(
        title='Funktsiooni f_vanus kontrollid',
        tests=[
            FunctionTest(
                title='Kas kuupäevaga 09.09.2000 on vanus õige?',
                name='f_vanus',
                arguments=['09.09.2000'],
                expected_value=23,
                points=12,
            ),
            FunctionTest(
                title='Kas kuupäevaga 01.01.2000 on vanus õige?',
                name='f_vanus',
                arguments=['01.01.2000'],
                expected_value=24,
                points=13,
            ),
        ]
    ),
    # 25p
    ChecksLayer(
        title='Funktsiooni f_klubiranking kontrollid',
        tests=[
            FunctionTest(
                title='Kas klubi Ajurebend ranking on õige?',
                name='f_klubiranking',
                arguments=[54],
                expected_value=1279.6,
                points=12,
            ),
            FunctionTest(
                title='Kas klubi Musta kivi kummardajad ranking on õige?',
                name='f_klubiranking',
                arguments=[59],
                expected_value=1407.0,
                points=13,
            ),
        ]
    ),
    # 25p
    ChecksLayer(
        title='Funktsiooni f_top10 kontrollid',
        tests=[
            FunctionTest(
                title='Kas on õige tulemuste arv?',
                name='f_top10',
                arguments=[44],
                expected_count=10,
                points=10,
            ),
            FunctionTest(
                title='Kas leidub mangija Murakas?',
                name='f_top10',
                where="mangija LIKE 'Murakas%'",
                arguments=[44],
                points=5,
            ),
            FunctionTest(
                title='Kas mangijal on õige paunktide arv?',
                name='f_top10',
                column_name='round(punkte,1)',
                where="mangija LIKE 'Muld%'",
                expected_value=5.5,
                arguments=[47],
                points=10,
            ),
        ]
    ),
    # 25p
    ChecksLayer(
        title='Protseduuri sp_uus_turniir kontrollid',
        tests=[
            ProcedureTest(
                title='Kas on õige lõppkuupäev ühe päevasel turniiril?',
                name='sp_uus_turniir',
                arguments=['Tartu Meister', '02.02.2022',1,'Tartu'],
                number_of_parameters=4,
                pre_query="DELETE FROM turniirid WHERE nimi='Tartu Meister'",
                after_query="select * from turniirid where nimi = 'Tartu Meister' and loppkuupaev = '02.02.2022'",
                points=11,
            ),
            DataTest(
                title='Kas turniiriga lisati ka uus asula?',
                name='asulad',
                where="nimi = 'Haapsalu'",
                points=2,
            ),
            ProcedureTest(
                title='Kas on õige lõppkuupäev kahe päevasel turniiril?',
                name='sp_uus_turniir',
                arguments=['Tartu Meister', '02.02.2022',2,'Tartu'],
                number_of_parameters=4,
                pre_query="DELETE FROM turniirid WHERE nimi='Tartu Meister'",
                after_query="select * from turniirid where nimi = 'Tartu Meister' and loppkuupaev = '03.02.2022'",
                points=12,
            ),
            DataTest(
                title='uus turniir',
                name='turniirid',
                column_name='count(*)',
                where="nimi='Tartu Meister'",
                expected_value=0,
            ),
        ]
    ),
]

    # Kodutöö 6 kontrollid
    # 10p + 10p
    ChecksLayer(
        title='Indeksite kontrollid',
        tests=[
            IndexTest(
                title='Kas on olemas ix_riiginimi?',
                name='ix_riiginimi',
                points=10,
            ),
            IndexTest(
                title='Kas on olemas ix_suurus?',
                name='ix_suurus',
                points=10,
            ),
        ],
    ),
    # 25p
    """
_user1 = 123456
_user2 = 123457
_partii_id = 123123
tests = [
    ChecksLayer(
        title='Trigger tg_partiiaeg kontrollid',
        tests=[
            TriggerTest(
                title='Kas on õige definitsioon tg_partiiaeg?',
                name='tg_partiiaeg',
                arguments=['UPDATE', 'INSERT'],
                action_timing='BEFORE',
                points=10,
            ),
            ExecuteLayer("ALTER TABLE public.partiid DISABLE TRIGGER ALL"),
            ExecuteLayer("ALTER TABLE public.partiid ENABLE TRIGGER tg_partiiaeg"),
            ExecuteLayer("ALTER TABLE public.isikud DISABLE TRIGGER ALL"),
            ExecuteLayer(f"DELETE FROM public.isikud WHERE id in ({_user1},{_user2})"),
            ExecuteLayer(f"INSERT INTO public.isikud (id, eesnimi, perenimi, klubis) VALUES ({_user1},'Man', 'Ka',54)"),
            DataTest(
                title='Kas testimiseks õnnestus lisada test_isik1?',
                name='isikud',
                where=f"isikud.id = {_user1} and EXISTS(SELECT * FROM information_schema.triggers WHERE trigger_name = 'tg_partiiaeg')",
                points=2,
            ),
            ExecuteLayer(f"INSERT INTO public.isikud (id, eesnimi, perenimi, klubis) VALUES ({_user2},'Kan', 'Ma',54)"),
            DataTest(
                title='Kas testimiseks õnnestus lisada test_isik2?',
                name='isikud',
                where=f"isikud.id = {_user2} and EXISTS(SELECT * FROM information_schema.triggers WHERE trigger_name = 'tg_partiiaeg')",
                points=2,
            ),
            ExecuteLayer(f"INSERT INTO public.partiid VALUES (44,'2023-04-22 17:45:24.000','2023-03-22 17:45:24.000',{_user1},{_user2},2,0, {_partii_id})", ),
            DataTest(
                title='Kas testimiseks lisatud isikute partii lõpphetk on õige?',
                name='partiid',
                column_name='COUNT(*)',
                where=f"lopphetk IS NULL and valge = {_user1} AND must = {_user2} and EXISTS(SELECT * FROM information_schema.triggers WHERE trigger_name = 'tg_partiiaeg')",
                expected_value=1,
                points=11,
            ),
        ],
    ),
]
"""
    # 25p
    ChecksLayer(
        title='Trigger tg_klubi_olemasolu kontrollid',
        tests=[
            ExecuteLayer("ALTER TABLE public.partiid DISABLE TRIGGER ALL"),
            ExecuteLayer("ALTER TABLE public.isikud DISABLE TRIGGER ALL"),
            ExecuteLayer("ALTER TABLE public.isikud ENABLE TRIGGER tg_klubi_olemasolu"),
            ExecuteLayer(f"DELETE FROM public.isikud WHERE id in ({_user1},{_user2})"),
            ExecuteLayer(f"INSERT INTO public.isikud (id, eesnimi, perenimi) VALUES ({_user1},'Man', 'Ka')"),
            TriggerTest(
                title='Kas on õige definitsioon tg_klubi_olemasolu?',
                name='tg_klubi_olemasolu',
                arguments=['UPDATE', 'INSERT'],
                action_timing='AFTER',
                points=10,
            ),
            DataTest(
                title='Kas on olemas klubi Klubitud?',
                name='klubid',
                column_name='COUNT(*)',
                where="lower(nimi) = 'klubitud' and EXISTS(SELECT * FROM information_schema.triggers WHERE trigger_name = 'tg_klubi_olemasolu')",
                points=3,
            ),
            DataTest(
                title='Kas testimiseks õnnestus lisada test_isik1?',
                name='isikud',
                where=f"public.isikud.id = {_user1} and EXISTS(SELECT * FROM information_schema.triggers WHERE trigger_name = 'tg_klubi_olemasolu')",
                points=2,
            ),
            DataTest(
                title='Kas lisatud isik on klubis Klubitud?',
                name='isikud',
                join=f"public.klubid ON public.klubid.id=isikud.klubis",
                where=f"public.isikud.id = {_user1} AND public.klubid.nimi LIKE '%ubitu%' and EXISTS(SELECT * FROM information_schema.triggers WHERE trigger_name = 'tg_klubi_olemasolu')",
                points=10,
            ),
        ]
    )
]
"""


from silmused.tests.TestDefinition import TestDefinition
from silmused.utils import list_to_string


class FunctionTest(TestDefinition):
    def __init__(self, name, title=None, arguments=None, column_name=None, where=None, description=None, expected_value=None,
                 expected_count=None, number_of_parameters=None, custom_feedback=None, points=0):
        super().__init__(
            name=name,
            title=title,
            where=where,
            points=points,
            arguments=arguments,
            description=description,
            expected_value=expected_value,
            expected_count=expected_count,
            custom_feedback=custom_feedback,
            query=f"SELECT {column_name if column_name is not None else '*'} FROM {name}({list_to_string(arguments if arguments is not None else '')})"  # TODO implement parameters,
        )
        if isinstance(expected_value, list):
            self.expected_value_list = True
            if isinstance(expected_value[0], str):
                self.expected_value_group = "strings"
            else:
                self.expected_value_group = "numbers"
                min_value = None
                max_value = None
                for value in expected_value:
                    if isinstance(value, str):
                        raise Exception('Ranged expected value cannot be a string')
                    if min_value is None:
                        min_value = value
                    elif value < min_value:
                        min_value = value
                    if max_value is None:
                        max_value = value
                    elif value > max_value:
                        max_value = value
                self.expected_min_value = min_value
                self.expected_max_value = max_value
        elif isinstance(expected_count, list):
            self.expected_value_list = True
            if isinstance(expected_count[0], str):
                self.expected_value_group = "strings"
            else:
                self.expected_value_group = "numbers"
                min_value = None
                max_value = None
                for value in expected_count:
                    if isinstance(value, str):
                        raise Exception('Ranged expected value cannot be a string')
                    if min_value is None:
                        min_value = value
                    elif value < min_value:
                        min_value = value
                    if max_value is None:
                        max_value = value
                    elif value > max_value:
                        max_value = value
                self.expected_min_value = min_value
                self.expected_max_value = max_value
        else:
            self.expected_value_list = False
            self.expected_value_group = "nothing"

        self.number_of_parameters = number_of_parameters

    def execute(self, cursor):

        # Check that function name exists
        test_function_exists_result = self.test_function_exists(cursor)
        if test_function_exists_result is not None:
            return test_function_exists_result

        # Check that type is correct
        test_function_type_result = self.test_function_type(cursor)
        if test_function_type_result is not None:
            return test_function_type_result

        # Check that function number of arguments is correct
        if self.number_of_parameters is not None:
            test_function_args_result = self.test_function_args(cursor)
            if test_function_args_result is not None:
                return test_function_args_result

        # TODO Add a test to check output arguments
        # cursor.execute("SELECT proargnames FROM pg_catalog.pg_proc WHERE proname='f_voit_viik_kaotus'")
        # test_tesult = cursor.fetchall()[0][0]
        # print(len(test_tesult)-(self.number_of_parameters if self.number_of_parameters is not None else 0))

        cursor.execute(self.query)
        result = cursor.fetchall()

        if self.expected_value is None:
            if self.expected_count is None:
                if self.custom_feedback is None:
                    return super().response(
                        len(result) > 0,
                        {"test_type": "function_test",
                         "test_key": "function_not_expected_value_not_expected_result_count_positive_feedback",
                         "params": [self.name, list_to_string(self.arguments)]},
                        {"test_type": "function_test",
                         "test_key": "function_not_expected_value_not_expected_result_count_negative_feedback",
                         "params": [self.name, list_to_string(self.arguments)]}
                    )
                else:
                    return super().response(
                        len(result) > 0,
                        {"test_type": "function_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                        {"test_type": "function_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                    )
            else:
                if self.expected_value_group == "numbers":
                    if self.custom_feedback is None:
                        return super().response(
                            self.expected_min_value <= len(result) <= self.expected_max_value,
                            {"test_type": "function_test",
                             "test_key": "function_expected_value_group_numbers_positive_feedback",
                             "params": [len(result), self.expected_min_value, self.expected_max_value]},
                            {"test_type": "function_test",
                             "test_key": "function_expected_value_group_numbers_negative_feedback",
                             "params": [len(result), self.expected_min_value, self.expected_max_value]},
                        )
                    else:
                        return super().response(
                            self.expected_min_value <= len(result) <= self.expected_max_value,
                            {"test_type": "function_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                            {"test_type": "function_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                        )
                elif self.expected_value_group == "strings":
                    if self.custom_feedback is None:
                        return super().response(
                            result[0][0] in self.expected_value,
                            {"test_type": "function_test",
                             "test_key": "function_expected_value_group_strings_positive_feedback",
                             "params": [str(result[0][0]), self.expected_value]},
                            {"test_type": "function_test",
                             "test_key": "function_expected_value_group_strings_negative_feedback",
                             "params": [str(result[0][0]), self.expected_value]},
                        )
                    else:
                        return super().response(
                            result[0][0] in self.expected_value,
                            {"test_type": "function_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                            {"test_type": "function_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                        )
                if self.custom_feedback is None:
                    return super().response(
                        len(result) == self.expected_count,
                        {"test_type": "function_test",
                         "test_key": "function_not_expected_value_expected_result_count_positive_feedback",
                         "params": [self.expected_count, self.name, list_to_string(self.arguments)]},
                        {"test_type": "function_test",
                         "test_key": "function_not_expected_value_expected_result_count_negative_feedback",
                         "params": [self.expected_count, self.name, list_to_string(self.arguments), len(result)]}
                    )
                else:
                    return super().response(
                        len(result) == self.expected_count,
                        {"test_type": "function_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                        {"test_type": "function_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                    )
        else:
            # TODO when type is decimal but we give float, we get an error
            # if type(result[0][0]) != type(self.expected_value):
            #     return super().response(
            #         False,
            #         'Correct',
            #         f"Expected type {type(self.expected_value)} but got {type(result[0][0])}",
            #     )
            if self.expected_value_list:
                if self.expected_value_group == "numbers":
                    if self.custom_feedback is None:
                        return super().response(
                            self.expected_min_value <= result[0][0] <= self.expected_max_value,
                            {"test_type": "function_test",
                             "test_key": "function_expected_value_group_numbers_positive_feedback",
                             "params": [str(result[0][0]), self.expected_min_value, self.expected_max_value]},
                            {"test_type": "function_test",
                             "test_key": "function_expected_value_group_numbers_negative_feedback",
                             "params": [str(result[0][0]), self.expected_min_value, self.expected_max_value]},
                        )
                    else:
                        return super().response(
                            self.expected_min_value <= result[0][0] <= self.expected_max_value,
                            {"test_type": "function_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                            {"test_type": "function_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                        )
                elif self.expected_value_group == "strings":
                    if self.custom_feedback is None:
                        return super().response(
                            result[0][0] in self.expected_value,
                            {"test_type": "function_test",
                             "test_key": "function_expected_value_group_strings_positive_feedback",
                             "params": [str(result[0][0]), self.expected_value]},
                            {"test_type": "function_test",
                             "test_key": "function_expected_value_group_strings_negative_feedback",
                             "params": [str(result[0][0]), self.expected_value]},
                        )
                    else:
                        return super().response(
                            result[0][0] in self.expected_value,
                            {"test_type": "function_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                            {"test_type": "function_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                        )
            if self.custom_feedback is None:
                return super().response(
                    str(result[0][0]) == str(self.expected_value),
                    {"test_type": "function_test",
                     "test_key": "function_expected_value_positive_feedback",
                     "params": [self.expected_value]},
                    {"test_type": "function_test",
                     "test_key": "function_expected_value_negative_feedback",
                     "params": [self.expected_value, result[0][0]]}
                )
            else:
                return super().response(
                    str(result[0][0]) == str(self.expected_value),
                    {"test_type": "function_test",
                     "test_key": "custom_feedback",
                     "params": [self.custom_feedback]},
                    {"test_type": "function_test",
                     "test_key": "custom_feedback",
                     "params": [self.custom_feedback]},
                )

    def test_function_exists(self, cursor):
        cursor.execute(f"SELECT * FROM pg_catalog.pg_proc WHERE proname='{self.name}'")
        if len(cursor.fetchall()) <= 0:
            if self.custom_feedback is None:
                return super().response(
                    False,
                    {"test_type": "function_test",
                     "test_key": "function_exists_positive_feedback",
                     "params": [self.name]},
                    {"test_type": "function_test",
                     "test_key": "function_exists_negative_feedback",
                     "params": [self.name]}
                )
            else:
                return super().response(
                    False,
                    {"test_type": "function_test",
                     "test_key": "custom_feedback",
                     "params": [self.custom_feedback]},
                    {"test_type": "function_test",
                     "test_key": "custom_feedback",
                     "params": [self.custom_feedback]},
                )
        return None

    def test_function_type(self, cursor):
        cursor.execute(
            f"SELECT routine_name FROM information_schema.routines WHERE routine_type = 'FUNCTION' AND routine_name='{self.name}'")
        if not len(cursor.fetchall()) > 0:
            if self.custom_feedback is None:
                return super().response(
                    False,
                    {"test_type": "function_test",
                     "test_key": "function_type_positive_feedback",
                     "params": [self.name]},
                    {"test_type": "function_test",
                     "test_key": "function_type_negative_feedback",
                     "params": [self.name]}
                )
            else:
                return super().response(
                    False,
                    {"test_type": "function_test",
                     "test_key": "custom_feedback",
                     "params": [self.custom_feedback]},
                    {"test_type": "function_test",
                     "test_key": "custom_feedback",
                     "params": [self.custom_feedback]},
                )
        return None

    def test_function_args(self, cursor):
        cursor.execute(f"SELECT pronargs FROM pg_catalog.pg_proc WHERE proname='{self.name}'")
        number_of_parameters_result = cursor.fetchall()[0][0]
        if not number_of_parameters_result == self.number_of_parameters:
            if self.custom_feedback is None:
                return super().response(
                    False,
                    {"test_type": "function_test",
                     "test_key": "function_parameters_amount_positive_feedback",
                     "params": [self.name]}
                    ,
                    {"test_type": "function_test",
                     "test_key": "function_parameters_amount_negative_feedback",
                     "params": [self.number_of_parameters, self.name, number_of_parameters_result]}
                )
            else:
                return super().response(
                    False,
                    {"test_type": "function_test",
                     "test_key": "custom_feedback",
                     "params": [self.custom_feedback]},
                    {"test_type": "function_test",
                     "test_key": "custom_feedback",
                     "params": [self.custom_feedback]},
                )
        return None



from silmused.ChecksLayer import ChecksLayer
from silmused.tests.QueryStructureTest import QueryStructureTest
from silmused.tests.QueryDataTest import QueryDataTest

tests = [
    ChecksLayer(
        title='Praktikum 3 Päringu 2 kontrollid',
        tests=[
            QueryStructureTest(
                title='Kas on olemas veerg pealkirja_algus?',
                name='query_test',
                column_name='pealkirja_algus',
                points=20,
            ),
            QueryStructureTest(
                title='Kas on olemas veerg arv?',
                name='query_test',
                column_name='arv',
                points=20,
            ),
            QueryDataTest(
                title='Kas on õige ridade arvuga tulemus?',
                name='query_test',
                column_name='COUNT(*)',
                expected_value=6,
                points=20,
            ),
            QueryDataTest(
                title='Kas leidub pealkirja algus Danc?',
                name='query_test',
                where='"Pealkirja algus"' + "='Danc'",
                points=20,
            ),
            QueryDataTest(
                title='Kas on õige arv "The " algusega pealkirju?',
                name='query_test',
                column_name='"pealkirja_algus"',
                where='arv=15',
                expected_value='The ',
                points=20,
            ),
        ]
    ),
]


from silmused.ChecksLayer import ChecksLayer
from silmused.tests.QueryStructureTest import QueryStructureTest
from silmused.tests.QueryDataTest import QueryDataTest

tests = [
    ChecksLayer(
        title='Ülesanne 4 Päringu kontrollid',
        tests=[
            QueryStructureTest(
                title='Kas on olemas veerg riikide arv?',
                name='query_test',
                column_name='riikide arv',
                points=20,
            ),
            QueryStructureTest(
                title='Kas on olemas veerg keskmine rahvaarv?',
                name='query_test',
                column_name='keskmine rahvaarv',
                points=20,
            ),
            QueryDataTest(
                title='Kas on õige ridade arvuga tulemus?',
                name='query_test',
                column_name='COUNT(*)',
                expected_value=1,
                points=20,
            ),
            QueryDataTest(
                title='Kas on õige riikide arv?',
                name='query_test',
                column_name='"riikide arv"',
                expected_value="3",
                points=20,
            ),
            QueryDataTest(
                title='Kas on õige keskmine?',
                name='query_test',
                column_name='"keskmine rahvaarv"',
                expected_value="1799472.333333333333",
                points=20,
            ),
        ]
    ),
]


from silmused.ChecksLayer import ChecksLayer
from silmused.tests.QueryStructureTest import QueryStructureTest
from silmused.tests.QueryDataTest import QueryDataTest

tests = [
    ChecksLayer(
        title='Praktikum 3 Päringu 1 kontrollid',
        tests=[
            QueryDataTest(
                title='Kas on õige tulemuste järjekord?',
                name='query_test',
                column_name='keel',
                where="test_id=1",
                expected_value="French",
                points=20,
            ),
            QueryDataTest(
                title='Kas on õige arv French keelelisi laule?',
                name='query_test',
                column_name='arv',
                where="keel='French'",
                expected_value="11",
                points=20,
            ),
        ]
    ),
]


from silmused.ChecksLayer import ChecksLayer
from silmused.tests.QueryStructureTest import QueryStructureTest
from silmused.tests.QueryDataTest import QueryDataTest

tests = [
    ChecksLayer(
        title='Ülesanne 2 Päringu kontrollid',
        tests=[
            QueryStructureTest(
                title='Kas on olemas veerg lavanimi?',
                name='query_test',
                column_name='lavanimi',
                points=10,
            ),
            QueryStructureTest(
                title='Kas on olemas veerg hetkevanus?',
                name='query_test',
                column_name='hetkevanus',
                points=10,
            ),
            QueryDataTest(
                title='Kas on õige ridade arvuga tulemus?',
                name='query_test',
                column_name='COUNT(*)',
                expected_value=12,
                points=30,
            ),
            QueryDataTest(
                title='Kas kõige noorem laulja on Iru?',
                name='query_test',
                column_name='lavanimi',
                where="test_id=1",
                expected_value="Iru",
                points=25,
            ),
            QueryDataTest(
                title='Kas Aiko vanus on õige?',
                name='query_test',
                column_name='hetkevanus',
                where="lavanimi='Aiko'",
                expected_value=[24,25],
                points=25,
            ),
        ]
    ),
]


from silmused.tests.TestDefinition import TestDefinition
from silmused.utils import list_to_string


class ProcedureTest(TestDefinition):
    def __init__(self, name, arguments, title=None, description=None, expected_value=None,
                 expected_count=None, number_of_parameters=None, pre_query=None, after_query=None,
                 custom_feedback=None, points=0):
        if after_query is None:
            raise Exception('Parameter "after_query" is required')

        super().__init__(
            name=name,
            title=title,
            points=points,
            arguments=arguments,
            description=description,
            expected_value=expected_value,
            expected_count=expected_count,
            pre_query=pre_query,
            query=f"CALL {name}({list_to_string(arguments)})",
            after_query=after_query,
            custom_feedback=custom_feedback
        )

        self.number_of_parameters = number_of_parameters

    def execute(self, cursor):

        # Check that procedure name exists
        test_procedure_exists_result = self.test_procedure_exists(cursor)
        if test_procedure_exists_result is not None:
            return test_procedure_exists_result

        # Check that type is correct
        test_procedure_type_result = self.test_procedure_type(cursor)
        if test_procedure_type_result is not None:
            return test_procedure_type_result

        # Check that procedure number of arguments is correct
        if self.number_of_parameters is not None:
            test_procedure_args_result = self.test_procedure_args(cursor)
            if test_procedure_args_result is not None:
                return test_procedure_args_result

        if self.pre_query is not None:
            cursor.execute(self.pre_query)

        cursor.execute(self.query)

        cursor.execute(self.after_query)

        result = cursor.fetchall()

        if self.expected_count is None:
            if self.custom_feedback is None:
                return super().response(
                    len(result) > 0,
                    {"test_type": "procedure_test",
                     "test_key": "procedure_not_expected_result_count_negative_feedback",
                     "params": [self.name, list_to_string(self.arguments)]},
                    {"test_type": "procedure_test",
                     "test_key": "procedure_not_expected_result_count_negative_feedback",
                     "params": [self.name, list_to_string(self.arguments)]}
                )
            else:
                return super().response(
                    len(result) > 0,
                    {"test_type": "index_test",
                     "test_key": "custom_feedback",
                     "params": [self.custom_feedback]},
                    {"test_type": "index_test",
                     "test_key": "custom_feedback",
                     "params": [self.custom_feedback]},
                )
        else:
            if self.custom_feedback is None:
                return super().response(
                    len(result) == self.expected_count,
                    {"test_type": "procedure_test",
                     "test_key": "procedure_expected_result_count_negative_feedback",
                     "params": [self.expected_count, self.name, list_to_string(self.arguments), len(result)]},
                    {"test_type": "procedure_test",
                     "test_key": "procedure_expected_result_count_negative_feedback",
                     "params": [self.expected_count, self.name, list_to_string(self.arguments), len(result)]}
                )
            else:
                return super().response(
                    len(result) == self.expected_count,
                    {"test_type": "index_test",
                     "test_key": "custom_feedback",
                     "params": [self.custom_feedback]},
                    {"test_type": "index_test",
                     "test_key": "custom_feedback",
                     "params": [self.custom_feedback]},
                )
        # return super().response(
        #    len(result) > 0,
        #    f"Correct count > 0 for procedure \"{self.name}({list_to_string(self.arguments)})\"",
        #    f"Expected count > 0 for procedure \"{self.name}({list_to_string(self.arguments)})\" but none was found",
        # )

    def test_procedure_exists(self, cursor):
        cursor.execute(f"SELECT * FROM pg_catalog.pg_proc WHERE proname='{self.name}'")
        if len(cursor.fetchall()) <= 0:
            if self.custom_feedback is None:
                return super().response(
                    False,
                    {"test_type": "procedure_test",
                     "test_key": "procedure_exists_positive_feedback",
                     "params": [self.name]},
                    {"test_type": "procedure_test",
                     "test_key": "procedure_exists_negative_feedback",
                     "params": [self.name]}
                )
            else:
                return super().response(
                    False,
                    {"test_type": "index_test",
                     "test_key": "custom_feedback",
                     "params": [self.custom_feedback]},
                    {"test_type": "index_test",
                     "test_key": "custom_feedback",
                     "params": [self.custom_feedback]},
                )
        return None

    def test_procedure_type(self, cursor):
        cursor.execute(
            f"SELECT routine_name FROM information_schema.routines WHERE routine_type = 'PROCEDURE' AND routine_name='{self.name}'")
        if not len(cursor.fetchall()) > 0:
            if self.custom_feedback is None:
                return super().response(
                    False,
                    {"test_type": "procedure_test",
                     "test_key": "procedure_type_positive_feedback",
                     "params": [self.name]},
                    {"test_type": "procedure_test",
                     "test_key": "procedure_type_negative_feedback",
                     "params": [self.name]}
                )
            else:
                return super().response(
                    False,
                    {"test_type": "index_test",
                     "test_key": "custom_feedback",
                     "params": [self.custom_feedback]},
                    {"test_type": "index_test",
                     "test_key": "custom_feedback",
                     "params": [self.custom_feedback]},
                )
        return None

    def test_procedure_args(self, cursor):
        cursor.execute(f"SELECT pronargs FROM pg_catalog.pg_proc WHERE proname='{self.name}'")
        number_of_parameters_result = cursor.fetchall()[0][0]
        if not number_of_parameters_result == self.number_of_parameters:
            if self.custom_feedback is None:
                return super().response(
                    False,
                    {"test_type": "procedure_test",
                     "test_key": "procedure_parameters_exists_positive_feedback",
                     "params": [self.name]},
                    {"test_type": "procedure_test",
                     "test_key": "procedure_parameters_exists_negative_feedback",
                     "params": [self.number_of_parameters, self.name, number_of_parameters_result]}
                )
            else:
                return super().response(
                    False,
                    {"test_type": "index_test",
                     "test_key": "custom_feedback",
                     "params": [self.custom_feedback]},
                    {"test_type": "index_test",
                     "test_key": "custom_feedback",
                     "params": [self.custom_feedback]},
                )
        return None



from silmused.ChecksLayer import ChecksLayer
from silmused.tests.QueryStructureTest import QueryStructureTest
from silmused.tests.QueryDataTest import QueryDataTest
tests = [
    ChecksLayer(
        title='Ülesanne 4 Päringu kontrollid',
        tests=[
            QueryStructureTest(
                title='Kas on olemas veerg ränkingu klass?',
                name='query_test',
                column_name="sünnikuu number",
                points=20,
            ),
            QueryStructureTest(
                title='Kas on olemas veerg arv?',
                name='query_test',
                column_name='arv',
                points=20,
            ),
            QueryStructureTest(
                title='Kas lahenduses on kasutatud keelatud elemente?',
                name='query_test',
                elements='LIMIT',
                should_exist=False,
                points=10,
            ),
            QueryDataTest(
                title='Kas on õige ridade arvuga tulemus?',
                name='query_test',
                column_name='COUNT(*)',
                expected_value=3,
                points=20,
            ),
            QueryDataTest(
                title='Kas on mais kõige vähem mängijaid?',
                name='query_test',
                column_name='arv',
                where='"sünnikuu number" = 5',
                expected_value=2,
                points=15,
            ),
            QueryDataTest(
                title='Kas on septembris õige arv mängijaid?',
                name='query_test',
                column_name='arv',
                where='"sünnikuu number" = 9',
                expected_value=3,
                points=15,
            ),
        ]
    ),
]


from silmused.ChecksLayer import ChecksLayer
from silmused.tests.QueryStructureTest import QueryStructureTest
from silmused.tests.QueryDataTest import QueryDataTest

tests = [
    ChecksLayer(
        title='Praktikum 4 ülesande 3 päringu kontrollid',
        tests=[
            QueryStructureTest(
                title='Kas on olemas veerg pealkiri?',
                name='query_test',
                column_name='pealkiri',
                points=20,
            ),
            QueryStructureTest(
                title='Kas on olemas veerg punktid_finaalis?',
                name='query_test',
                column_name='punktid_finaalis',
                points=20,
            ),
            QueryStructureTest(
                title='Kas on olemas veerg koht_finaalis?',
                name='query_test',
                column_name='koht_finaalis',
                points=20,
            ),
            QueryDataTest(
                title='Kas on õige ridade arvuga tulemus?',
                name='query_test',
                column_name='COUNT(*)',
                expected_value=10,
                points=20,
            ),
            QueryDataTest(
                title='Kas leidub laul Kuula?',
                name='query_test',
                where="pealkiri='Kuula'",
                points=10,
            ),
            QueryDataTest(
                title='Kas puuduvate väärtustega read on kadunud?',
                name='query_test',
                column_name='COUNT(*)',
                where="koht_finaalis is NULL",
                expected_value=0,
                points=10,
            ),
        ]
    ),
]


from silmused.ChecksLayer import ChecksLayer
from silmused.tests.QueryStructureTest import QueryStructureTest
from silmused.tests.QueryDataTest import QueryDataTest
tests = [
    ChecksLayer(
        title='Ülesanne 3 Päringu kontrollid',
        tests=[
            QueryStructureTest(
                title='Kas on olemas veerg eesnimi?',
                name='query_test',
                column_name="eesnimi",
                points=10,
            ),
            QueryStructureTest(
                title='Kas on olemas veerg perenimi?',
                name='query_test',
                column_name="perenimi",
                points=10,
            ),
            QueryStructureTest(
                title='Kas on olemas veerg keskmine tulemus?',
                name='query_test',
                column_name="keskmine tulemus",
                points=10,
            ),
            QueryStructureTest(
                title='Kas on olemas veerg nupu värv?',
                name='query_test',
                column_name="nupu värv",
                points=10,
            ),
            QueryDataTest(
                title='Kas on õige ridade arvuga tulemus?',
                name='query_test',
                column_name='COUNT(*)',
                expected_value=160,
                points=10,
            ),
            QueryDataTest(
                title='Kas on NULL tulemus mustaga mängijal Heli Jälg?',
                name='query_test',
                column_name='"keskmine tulemus"',
                where="eesnimi = 'Heli' and perenimi = 'Jälg'",
                expected_value='NULL',
                points=20,
            ),
            QueryDataTest(
                title='Kas on õige keskmine valgega mängijal Kalev Jõud?',
                name='query_test',
                column_name='round("keskmine tulemus",0)',
                where="eesnimi = 'Kalev' and test_id=54",
                expected_value=1,
                points=20,
            ),
            QueryDataTest(
                title='Kas on õige keskmine eespool mängijal Andrei Sosnov?',
                name='query_test',
                column_name='"nupu värv"',
                where="test_id=7",
                expected_value='must',
                points=10,
            ),
        ]
    ),
]


import json
import os
import subprocess
import sys
import re

import psycopg2
from uuid import uuid4
from datetime import datetime
from silmused.version import __version__
from silmused.Translator import Translator
from psycopg2.extensions import ISOLATION_LEVEL_AUTOCOMMIT


def _results_to_string(results):
    string = ''
    for result in results:
        if result.get('type') != 'execution' and result.get('message') is not None:
            string += str(result.get('message')) + '\n'

    print(string)


class Runner:
    def __init__(self, backup_file_path, tests, lang='en', test_name='', db_user='postgres', db_host='localhost',
                 db_password='postgres', db_port='5432', test_query='test', query_sql='', encoding=None):
        self.file_path = backup_file_path
        self.tests = tests
        self.test_name = test_name
        self.db_user = db_user
        self.db_name = ''
        self.db_password = db_password
        self.db_host = db_host
        self.db_port = db_port
        self.test_query = test_query
        self.query_sql = query_sql
        self.encoding = encoding

        self.db_name = f"db{'_' + self.test_name if self.test_name != '' else ''}_{self.file_path.split('/')[-1].split('.')[0]}_{str(uuid4()).replace('-', '_')}"

        self.results = []
        self.translator = Translator(locale=lang)

        if self.test_query == 'test':
            if self._file_is_valid_pg_dump():
                self._create_db_from_psql_dump()
                self.results = self._run_tests()
            elif self._file_is_valid_pg_insert():
                self._create_db_from_psql_insert()
                self.results = self._run_tests()
            else:
                print(self.translator.translate("sys_fail", "incorrect_dump_file"))
        elif self.test_query == 'query':
            if len(self.query_sql) == 0:
                print(self.translator.translate("sys_fail", "empty_query_file"))
            elif self._file_is_valid_pg_dump():
                self._create_db_from_psql_dump()
                self._create_query_view()
                self.results = self._run_tests()
            elif self._file_is_valid_pg_insert():
                self._create_db_from_psql_insert()
                self._create_query_view()
                self.results = self._run_tests()
            else:
                print(self.translator.translate("sys_fail", "incorrect_dump_file"))
        else:
            print(self.translator.translate("sys_fail", "incorrect_test_format"))

    def _file_is_valid_pg_dump(self):
        if not os.path.isfile(self.file_path):
            return False

        try:
            with open(self.file_path, 'rb') as file:
                magic_number = file.read(5)
                return magic_number == b'PGDMP'
        except IOError:
            return False

    def _create_db_from_psql_dump(self):
        subprocess.run(["createdb", "-U", self.db_user, self.db_name])

        subprocess.run(
            ["psql", "-U", self.db_user, "-d", self.db_name, "-c", "DROP SCHEMA public CASCADE;"],
            stdout=subprocess.DEVNULL,
            stderr=subprocess.DEVNULL
        )

        subprocess.run(
            ["pg_restore", "-U", self.db_user, "-d", self.db_name, "--no-owner", "--no-acl", "-1", self.file_path]
        )

    def _file_is_valid_pg_insert(self):
        if not os.path.isfile(self.file_path):
            return False

        if not self.file_path.lower().endswith('.sql'):
            return False

        #if len(self.encoding) > 0:
        try:
            with open(self.file_path, 'r', encoding=self.encoding) as file:
                lines = file.readlines()
                return any(line.strip().startswith("INSERT") for line in lines)
        except IOError:
            return False


    def _create_db_from_psql_insert(self):
        connection = self._connect(db_name='postgres')
        connection.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
        cursor = connection.cursor()

        try:
            cursor.execute(f"CREATE DATABASE {self.db_name};")
        except Exception as exception:
            print(f"CREATE DATABASE failed: {exception}")
        finally:
            cursor.close()
            connection.close()

        connection = self._connect()
        cursor = connection.cursor()

        try:
            #if len(self.encoding) > 0:
            with open(self.file_path, 'r', encoding=self.encoding) as file:
                sql_script = file.read()

            # Postgre 17.6+ added rstrict/unrestrict to dump files, if these are not removed then psycopg2 script execution fails
            if re.findall(r"\\restrict", sql_script):
                sql_script = re.sub(r"(--.*$)|(^\\restrict.*$)|(^\\unrestrict.*$)", "", sql_script,
                                    flags=re.MULTILINE)
            if re.findall(r".*CREATE SCHEMA public;.*", sql_script):
                cursor.execute('DROP SCHEMA IF EXISTS public')
            cursor.execute(sql_script)
            connection.commit()
        except Exception as exception:
            print(f"Sql INSERT import failed: {exception}")
        finally:
            cursor.close()
            connection.close()

    def _run_tests(self):
        connection = self._connect()
        cursor = connection.cursor()

        try:
            results = []

            for test in self.tests:
                results.append(test.run(cursor))

            return results
        except Exception as exception:
            print(f"Sql TEST RUN failed: {exception}")
        finally:
            cursor.close()
            connection.close()

    def _create_query_view(self):
        connection = self._connect()
        cursor = connection.cursor()
        try:
            cursor.execute("CREATE VIEW query_view AS " + self.query_sql)
            cursor.execute("create table query_test as select * from query_view limit 0;")
            cursor.execute("alter table query_test add column test_id serial;")
            cursor.execute("insert into query_test select * from query_view;")
            connection.commit()
        except Exception as exception:
            print(f"Running SQL failed: {exception}")
        finally:
            cursor.close()
            connection.close()

    def _connect(self, db_name=None):
        connection_layer = psycopg2.connect(
            host=self.db_host,
            port=self.db_port,
            database=self.db_name if db_name is None else db_name,
            user=self.db_user,
            password=self.db_password
        )

        return connection_layer

    def _message_to_feedback(self, message):
        feedback = ''
        feedback_params = [key for key, value in message.items() if key not in ['test_type', 'test_key']]
        if len(feedback_params) > 0:
            params = message['params']
            if len(params) == 0:
                feedback = self.translator.translate(message['test_type'], message['test_key'])

            elif len(params) == 1:
                feedback = self.translator.translate(message['test_type'], message['test_key'],
                                param1 = params[0] if type(params[0]) != list else self._translate_param_separation(params[0]))
            elif len(params) == 2:
                feedback = self.translator.translate(message['test_type'], message['test_key'],
                                                     param1=params[0], param2=params[1])
            elif len(params) == 3:
                feedback = self.translator.translate(message['test_type'], message['test_key'],
                                                     param1=params[0], param2=params[1], param3=params[2])
            elif len(params) == 4:
                feedback = self.translator.translate(message['test_type'], message['test_key'],
                                                     param1=params[0], param2=params[1], param3=params[2],
                                                     param4=params[3])
            elif len(params) == 5:
                feedback = self.translator.translate(message['test_type'], message['test_key'],
                                                     param1=params[0], param2=params[1], param3=params[2],
                                                     param4=params[3], param5=params[4])
            else:
                feedback = "Params were given, but there is more than 5"

        return feedback

    # This is used, when the input param is in a list, but it would be better not to output as a python list, but OR list
    def _translate_param_separation(self, param_list):
        or_lang = {'en': 'or', 'et': 'või'}
        output = ''
        for (index, param) in enumerate(param_list):
            operator = '' if index == 0 else f"' {or_lang[self.translator.locale]} '"
            output += f"{operator}{param}"
        return output

    def _checks_to_object(self, checks):
        outputs = []
        output_pass = True


        # TODO should be recursive
        points_max = 0
        points_actual = 0

        for check in checks:
            if check.get('type') == 'execution':
                continue
            elif check.get('type') == 'message':
                outputs.append({
                    "title": str(check.get('message')),
                    "status": 'PASS'
                })
                continue

            points = check.get('points') if check.get('points') is not None else 0
            points_max += points
            points_actual += points if check.get('is_success') else 0

            output = {}
            output["title"] = check.get('title')
            output["status"] = 'PASS' if check.get('is_success') else 'FAIL'
            output_pass = True if check.get('is_success') and output_pass else False
            if not check.get('is_success') and not check.get('is_sys_fail'):
                output["feedback"] = self._message_to_feedback(check.get('message'))
            elif check.get('is_sys_fail'):
                output["feedback"] = str(check.get('message'))
            else:
                output["feedback"] = ''
            outputs.append(output)

        return points_max, points_actual, outputs, output_pass

    def _results_to_object(self):
        tests = []

        points_max = 0
        points_actual = 0

        for result in self.results:
            if result.get('type') == 'execution':
                continue
            elif result.get('type') == 'message':
                tests.append({
                    "title": str(result.get('message')),
                    "status": 'PASS'
                })
                continue

            output = {}

            if result.get('type') == 'checks_layer':
                checks_points_max, checks_points_actual, checks_outputs, output_pass = self._checks_to_object(result.get('checks'))
                points_max += checks_points_max
                points_actual += checks_points_actual
                output["checks"] = checks_outputs
                output["status"] = 'PASS' if output_pass else 'FAIL'
            else:
                points = result.get('points') if result.get('points') is not None else 0
                points_max += points
                points_actual += points if result.get('is_success') else 0
                output["status"] = 'PASS' if result.get('is_success') else 'FAIL'

            output["title"] = result.get('title')
            if result.get('message') is not None:
                output["exception_message"] = str(result.get('message'))

            tests.append(output)
        return tests, points_max, points_actual

    def get_results(self):
        try:
            tests, points_max, points_actual = self._results_to_object()
            # TODO Put all logic in points variable
            praks = True if len(tests) > 0 and points_max == 0 and points_actual == 0 else False
            if praks:
                points_max = 1
                points_actual = 1

            if points_max != 0:
                output = {
                    "result_type": "OK_V3",
                    "points": round(100 * points_actual / points_max),
                    "producer": f"silmused {__version__}",
                    "finished_at": datetime.utcnow().strftime("%Y-%m-%dT%H:%M:%SZ"),
                    "tests": tests
                }
            else:
                output = {
                    "result_type": "OK_V3",
                    "points": 0,
                    "producer": f"silmused {__version__}",
                    "finished_at": datetime.utcnow().strftime("%Y-%m-%dT%H:%M:%SZ"),
                    "tests": tests
                }
            return json.dumps(output, ensure_ascii=False)
        except:
            print(sys.exc_info())
            return json.dumps({
              "result_type": "OK_V3",
              "producer": f"silmused {__version__}",
              "finished_at": datetime.utcnow().strftime("%Y-%m-%dT%H:%M:%SZ"),
              "points": 0,
            }, ensure_ascii=False)


import sys
import re


# TODO this should not be usable without a child
class TestDefinition:
    def __init__(self, name, points, title='', where=None, join=None, column_name=None, should_exist=True, query='',
                 description=None, arguments=None, expected_value=None, expected_character_maximum_length=None,
                 expected_type=None, expected_count=None, pre_query=None, after_query=None, custom_feedback=None,
                 elements=None, column_name_fallback=None, expected_value_query=None):
        if arguments is not None and not isinstance(arguments, list):
            raise Exception('Parameter "arguments" must be a list')

        #if expected_count is not None and (not isinstance(expected_count, int) or not isinstance(expected_count, list)):
        #    raise Exception('Parameter "expected_count" must be an integer or list')

        if not isinstance(points, int) and not isinstance(points, float):
            raise Exception('Parameter "points" must be either an integer or a float')

        if expected_value is not None and expected_count is not None:
            raise Exception('Both expected_value and check_count cannot be specified in a single test')

        query_builder = query

        # TODO right now a single join is possible (without a hack)
        if join is not None:
            query_builder += f" JOIN {join}"

        if where is not None:
            query_builder += f" WHERE ({where})"

        self.title = title
        self.points = points
        self.name = name
        self.column_name = column_name
        self.description = description
        self.arguments = arguments  # TODO arguments could be a class
        self.expected_value = expected_value
        self.expected_character_maximum_length = expected_character_maximum_length
        self.expected_type = expected_type
        self.expected_count = expected_count
        self.query = query_builder
        self.pre_query = pre_query
        self.after_query = after_query
        self.should_exist = should_exist  # TODO should be renamed to something more descriptive (should_be_false/falsy)
        self.custom_feedback = custom_feedback
        self.elements = elements
        self.column_name_fallback = column_name_fallback
        self.expected_value_query = expected_value_query

    # TODO should be callable only inside the scope
    def execute(self, cursor):
        raise NotImplementedError('Method "execute" not implemented')

    def run(self, cursor):
        try:
            # TODO could executing of pre and/or after queries be handled here?
            # print(self.query)
            return self.execute(cursor)
        except:
            # TODO better handler for rollback?
            #print(sys.exc_info())
            cursor.execute('ROLLBACK')
            if 'UndefinedColumn' in str(sys.exc_info()[0]):
                return self._undefined_column_error_feedback(str(sys.exc_info()[1]))
            if 'UndefinedTable' in str(sys.exc_info()[0]):
                return self._undefined_table_error_feedback(str(sys.exc_info()[1]))
            if 'AmbiguousColumn' in str(sys.exc_info()[0]):
                return self._ambiguous_column_error_feedback(str(sys.exc_info()[1]))
            if 'UndefinedFunction' in str(sys.exc_info()[0]):
                return self._undefined_function_error_feedback(str(sys.exc_info()[1]))
            if 'IndexError' in str(sys.exc_info()[0]):
                return self._index_error()
            return self.response(
                False,
                message_failure=sys.exc_info(),
                is_sys_fail=True
            )

    # TODO should be callable only inside the scope
    def response(self, is_success, message_success=None, message_failure=None, points=None, is_sys_fail=None):
        if is_success:
            message_statement = 'Correct' if message_success is None else message_success
        else:
            message_statement = 'Wrong' if message_failure is None else message_failure

        return {
            'is_success': is_success,
            'message': message_statement,
            'points': points if points is not None else self.points,
            'description': self.description,
            'query': self.query,
            'pre_query': self.pre_query,
            'after_query': self.after_query,
            'should_exist': self.should_exist,
            'title': self.title,
            'is_sys_fail': is_sys_fail,
        }

    def _undefined_column_error_feedback(self, sysfeedback):
        split_sys_feedback = sysfeedback.split('"')

        return self.response(
            False,
            '',
            {"test_type": "sys_fail",
             "test_key": "undefined_column",
             "params": [split_sys_feedback[1]]},
        )

    def _undefined_table_error_feedback(self, sysfeedback):
        split_sys_feedback = sysfeedback.split('"')

        return self.response(
            False,
            '',
            {"test_type": "sys_fail",
             "test_key": "undefined_table",
             "params": [split_sys_feedback[1]]},
        )

    def _ambiguous_column_error_feedback(self, sysfeedback):
        split_sys_feedback = sysfeedback.split('"')
        # print(sysfeedback)
        return self.response(
            False,
            '',
            {"test_type": "sys_fail",
             "test_key": "ambiguous_column",
             "params": [split_sys_feedback[1]]},
        )

    def _undefined_function_error_feedback(self, sysfeedback):
        if 'round' in sysfeedback:
            pattern = r'SELECT round\((.*?),[0-9]\)'
            match = re.findall(pattern, sysfeedback, re.IGNORECASE)
            if len(match) > 0:
                return self.response(
                    False,
                    '',
                    {"test_type": "sys_fail",
                     "test_key": "undefined_function_round",
                     "params": [match[0]]},
                )

        return self.response(
                False,
                message_failure=sysfeedback,
                is_sys_fail=True
            )

    def _index_error(self):
        return self.response(
            False,
            '',
            {"test_type": "sys_fail",
             "test_key": "index_error",
             "params": []},
        )



import sys

from silmused.Runner import Runner

'''
CLI to integrate the solution with e.g. a Docker sultion.

NB! The tests file must include an array with the key "tests"!!
'''


def main():
    sql_file_path = sys.argv[1]
    tests_path = sys.argv[2]
    db_user = sys.argv[3] if len(sys.argv) > 3 else 'postgres'
    db_host = sys.argv[4] if len(sys.argv) > 4 else 'localhost'
    db_port = sys.argv[5] if len(sys.argv) > 5 else '5432'
    db_password = sys.argv[6] if len(sys.argv) > 6 else 'postgresql'
    lang = sys.argv[7] if len(sys.argv) > 7 else 'en'
    test_query = sys.argv[8] if len(sys.argv) > 8 else 'test'
    querydb_path = sys.argv[9] if len(sys.argv) > 9 else ''
    sql_file_encoding = sys.argv[10] if len(sys.argv) > 10 else None

    with open(tests_path, 'r') as file:
        tests_file_content = file.read()
    local_vars = {}
    exec(tests_file_content, globals(), local_vars)
    tests = local_vars.get('tests')

    query_sql = ''
    if test_query == 'query':
        with open(sql_file_path, 'r') as file:
            query_sql = file.read()
        sql_file_path = querydb_path

    r=Runner(sql_file_path, tests, db_user=db_user, db_host=db_host, db_port=db_port, db_password=db_password, lang=lang,
             test_query=test_query, query_sql=query_sql, encoding=sql_file_encoding)
    print(r.get_results())


from silmused.tests.TestDefinition import TestDefinition
from silmused.utils import list_to_string


class StructureTest(TestDefinition):
    def __init__(self, name, title=None, column_name=None, arguments=None, expected_value=None, expected_type=None,
                 expected_character_maximum_length=None, should_exist=True, where=None,
                 description=None, custom_feedback=None, points=0):
        if column_name is None and expected_type is not None:
            raise Exception('Expected type needs a column to be set!')

        query = f"SELECT {list_to_string(arguments)[1:-1] if arguments is not None else '*'} FROM information_schema.columns WHERE table_name = '{name}'"

        if column_name is not None:
            if type(column_name) == str:
                query += f" AND column_name = '{column_name}'"
            elif type(column_name) == list:
                for (index, c_name) in enumerate(column_name):
                    operator = 'AND (' if index == 0 else 'OR'
                    query += f" {operator} column_name = '{c_name}'"
                query += ")"
            else:
                raise AttributeError('Parameter column_name must be list or string')

        super().__init__(
            name=name,
            title=title,
            where=where,
            points=points,
            arguments=arguments,
            description=description,
            expected_value=expected_value,
            expected_type=expected_type,
            expected_character_maximum_length=expected_character_maximum_length,
            query=query,
            should_exist=should_exist,
            custom_feedback=custom_feedback
        )

        self.column_name = column_name
        self.where = where

    def execute(self, cursor):
        if isinstance(self.column_name, list):
            self.query = self._check_separately_for_all_columns(cursor)
        cursor.execute(self.query)
        result = cursor.fetchall()
        test_type_result = self.test_type(result)
        if test_type_result is not None:
            return test_type_result

        if self.expected_value is not None:
            # TODO Is this needed?
            # What is this test checking? result 0,0 is database name, maybe this should check if the value is found in result?
            if self.should_exist:
                if self.custom_feedback is None:
                # print(self.title, result[0][0])
                    return super().response(
                        len(result) != 0 and result[0][0] == self.expected_value,
                        {"test_type": "structure_test",
                         "test_key": "expected_value_should_exist_positive_feedback",
                         "params": [self.query]},
                        {"test_type": "structure_test",
                         "test_key": "expected_value_should_exist_negative_feedback",
                         "params": [self.query, self.expected_value]},
                    )
                else:
                    return super().response(
                        len(result) != 0 and result[0][0] == self.expected_value,
                        {"test_type": "structure_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                        {"test_type": "structure_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                    )
            else:
                if self.custom_feedback is None:
                # print(self.title, result[0][0])
                    return super().response(
                        len(result) == 0 or result[0][0] != self.expected_value,
                        {"test_type": "structure_test",
                         "test_key": "expected_value_should_not_exist_positive_feedback",
                         "params": [self.query]},
                        {"test_type": "structure_test",
                         "test_key": "expected_value_should_not_exist_negative_feedback",
                         "params": [self.query, self.expected_value]},
                    )
                else:
                    return super().response(
                        len(result) == 0 or result[0][0] != self.expected_value,
                        {"test_type": "structure_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                        {"test_type": "structure_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                    )
        else:
            if self.should_exist:
                if self.column_name is None:
                    if self.custom_feedback is None:
                        return super().response(
                            len(result) > 0,
                            {"test_type": "structure_test",
                             "test_key": "table_should_exist_positive_feedback",
                             "params": [self.name]},
                            {"test_type": "structure_test",
                             "test_key": "table_should_exist_negative_feedback",
                             "params": [self.name]},
                        )
                    else:
                        return super().response(
                            len(result) > 0,
                            {"test_type": "structure_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                            {"test_type": "structure_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                        )
                else:
                    if self.custom_feedback is None:
                        return super().response(
                            len(result) > 0,
                            {"test_type": "structure_test",
                             "test_key": "column_should_exist_positive_feedback",
                             "params": [self.column_name, self.name]},
                            {"test_type": "structure_test",
                             "test_key": "column_should_exist_negative_feedback",
                             "params": [self.column_name, self.name]},
                        )
                    else:
                        return super().response(
                            len(result) > 0,
                            {"test_type": "structure_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                            {"test_type": "structure_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                        )
            else:
                if self.column_name is None:
                    if self.custom_feedback is None:
                        return super().response(
                            len(result) == 0,
                            {"test_type": "structure_test",
                             "test_key": "table_should_not_exist_positive_feedback",
                             "params": [self.name]},
                            {"test_type": "structure_test",
                             "test_key": "table_should_not_exist_negative_feedback",
                             "params": [self.name]},
                        )
                    else:
                        return super().response(
                            len(result) == 0,
                            {"test_type": "structure_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                            {"test_type": "structure_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                        )
                else:
                    if self.custom_feedback is None:
                        return super().response(
                            len(result) == 0,
                            {"test_type": "structure_test",
                             "test_key": "column_should_not_exist_positive_feedback",
                             "params": [self.column_name, self.name]},
                            {"test_type": "structure_test",
                             "test_key": "column_should_not_exist_negative_feedback",
                             "params": [self.column_name, self.name]},
                        )
                    else:
                        return super().response(
                            len(result) == 0,
                            {"test_type": "structure_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                            {"test_type": "structure_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                        )

    def test_type(self, result):
        if self.expected_type is None:
            return None

        type = result[0][7]
        character_maximum_length = result[0][8]
        is_type_correct = True
        is_character_maximum_length_correct = True

        if self.expected_type == 'float':
            if type not in ('float', 'double', 'decimal'):
                is_type_correct = False
        elif self.expected_type == 'integer':
            if type not in ('tinyint', 'smallint', 'mediumint', 'int', 'bigint', 'integer'):
                is_type_correct = False
        elif self.expected_type == 'varchar':
            if type != 'character varying':
                is_type_correct = False
            elif self.expected_character_maximum_length is not None and self.expected_character_maximum_length != character_maximum_length:
                is_character_maximum_length_correct = False
        elif self.expected_type == 'text':
            if type != 'text':
                is_type_correct = False
        elif self.expected_type == 'boolean':
            if type != 'boolean':
                is_type_correct = False

        if is_type_correct is False or is_character_maximum_length_correct is False:
            if self.custom_feedback is None:
                return super().response(
                    False,
                    None,
                    {"test_type": "structure_test",
                     "test_key": "expected_character_maximum_length_type_check_negative_feedback",
                     "params": [self.expected_character_maximum_length, character_maximum_length, self.name, self.column_name]}
                    if is_character_maximum_length_correct is False
                    else {"test_type": "structure_test",
                          "test_key": "expected_type_check_negative_feedback",
                          "params": [self.expected_type, type, self.name, self.column_name]},
                )
            else:
                return super().response(
                    False,
                    {"test_type": "structure_test",
                     "test_key": "custom_feedback",
                     "params": [self.custom_feedback]},
                    {"test_type": "structure_test",
                     "test_key": "custom_feedback",
                     "params": [self.custom_feedback]},
                )

        return None

    def _check_separately_for_all_columns(self, cursor):
        found = []
        for column in self.column_name:
            query = f"SELECT * FROM information_schema.columns WHERE table_name = '{self.name}' AND column_name = '{column}'"
            cursor.execute(query)
            result = cursor.fetchall()
            if len(result) > 0:
                found.append(column)
        if len(found) == 0: return self.query
        query = f"SELECT * FROM information_schema.columns WHERE table_name = '{self.name}'"
        self.column_name = found
        for (index, c_name) in enumerate(self.column_name):
            operator = 'AND (' if index == 0 else 'OR'
            query += f" {operator} column_name = '{c_name}'"
        query += ")"
        return query



from silmused.ChecksLayer import ChecksLayer
from silmused.tests.QueryStructureTest import QueryStructureTest
from silmused.tests.QueryDataTest import QueryDataTest

tests = [
    ChecksLayer(
        title='Ülesanne 1 Päringu kontrollid',
        tests=[
            QueryStructureTest(
                title='Kas on olemas veerg eesnimi?',
                name='test',
                column_name='eesnimi',
            ),
            QueryStructureTest(
                title='Kas on olemas veerg perenimi?',
                name='test',
                column_name='perenimi',
            ),
            QueryDataTest(
                title='Kas on õige ridade arvuga tulemus?',
                name='test',
                column_name='COUNT(*)',
                expected_value=5,
            ),
            QueryDataTest(
                title='Kas on olemas Sander Saabas?',
                name='test',
                column_name='eesnimi1',
                where="perenimi='Saabas'",
            ),
        ]
    ),
]
"""
tests=[
    ChecksLayer(
        title='Ülesanne 2 Päringu kontrollid',
        tests=[
            QueryStructureTest(
                title='Kas on olemas veerg eesnimi?',
                name='test',
                column_name='eesnimi',
            ),
            QueryStructureTest(
                title='Kas on olemas veerg perenimi?',
                name='test',
                column_name='perenimi',
            ),
            QueryStructureTest(
                title='Kas on olemas veerg sugu?',
                name='test',
                column_name='sugu',
            ),
            QueryStructureTest(
                title='Kas on olemas veerg vanus?',
                name='test',
                column_name='vanus',
            ),
            QueryDataTest(
                title='Kas on õige ridade arvuga tulemus?',
                name='test',
                column_name='COUNT(*)',
                expected_value=4,
            ),
            QueryDataTest(
                title='Kas on olemas Arvo Mets õige vanusega?',
                name='test',
                where="eesnimi = 'Arvo' and perenimi = 'Mets' and vanus = (select extract( year from age(current_date,synniaeg)) from isikud where eesnimi = 'Arvo' and perenimi = 'Mets')",
            ),
        ]
    ),
]
"""


from silmused.ChecksLayer import ChecksLayer
from silmused.tests.QueryStructureTest import QueryStructureTest
from silmused.tests.QueryDataTest import QueryDataTest

tests = [
    ChecksLayer(
        title='Praktikum 6 ülesande 2 päringu kontrollid',
        tests=[
            QueryStructureTest(
                title='Kas on olemas veerg aasta?',
                name='query_test',
                column_name='aasta',
                points=10,
            ),
            QueryStructureTest(
                title='Kas on olemas veerg esineja?',
                name='query_test',
                column_name='esineja',
                points=10,
            ),
            QueryStructureTest(
                title='Kas on olemas veerg laul?',
                name='query_test',
                column_name='laul',
                points=10,
            ),
            QueryStructureTest(
                title='Kas on olemas veerg keel?',
                name='query_test',
                column_name='keel',
                points=10,
            ),
            QueryStructureTest(
                title='Kas on olemas veerg punktid?',
                name='query_test',
                column_name='punktid',
                points=10,
            ),
            QueryStructureTest(
                title='Kas on olemas veerg koht?',
                name='query_test',
                column_name='koht',
                points=10,
            ),
            QueryDataTest(
                title='Kas on õige ridade arvuga tulemus?',
                name='query_test',
                column_name='COUNT(*)',
                expected_value=19,
                points=20,
            ),
            QueryDataTest(
                title='Kas on õige järjekord?',
                name='query_test',
                column_name='aasta',
                where="test_id=4",
                expected_value='2011',
                points=10,
            ),
            QueryDataTest(
                title='Kas on punktid puudu esinejal Manpower?',
                name='query_test',
                column_name='punktid',
                where="esineja='Manpower 4'",
                expected_value='None',
                points=10,
            ),
        ]
    ),
]


from silmused.ChecksLayer import ChecksLayer
from silmused.tests.QueryStructureTest import QueryStructureTest
from silmused.tests.QueryDataTest import QueryDataTest

tests = [
    ChecksLayer(
        title='Ülesanne 3 Päringu kontrollid',
        tests=[
            QueryStructureTest(
                title='Kas on olemas veerg riik_id?',
                name='query_test',
                column_name='riik_id',
                points=10,
            ),
            QueryStructureTest(
                title='Kas on olemas veerg arv?',
                name='query_test',
                column_name='arv',
                points=10,
            ),
            QueryDataTest(
                title='Kas on õige ridade arvuga tulemus?',
                name='query_test',
                column_name='COUNT(*)',
                expected_value=4,
                points=20,
            ),
            QueryDataTest(
                title='Kas on õige riigi id esimesel kohal?',
                name='query_test',
                column_name='riik_id',
                where="test_id=1",
                expected_value="3",
                points=30,
            ),
            QueryDataTest(
                title='Kas on õige arv riigil id-ga 2?',
                name='query_test',
                column_name='arv',
                where="riik_id=2",
                expected_value="6",
                points=30,
            ),
        ]
    ),
]


from silmused.ChecksLayer import ChecksLayer
from silmused.tests.QueryStructureTest import QueryStructureTest
from silmused.tests.QueryDataTest import QueryDataTest
tests = [
    ChecksLayer(
        title='Ülesanne 1 Päringu kontrollid',
        tests=[
            QueryStructureTest(
                title='Kas on olemas veerg nimi?',
                name='query_test',
                column_name="nimi",
                points=5,
            ),
            QueryStructureTest(
                title='Kas on olemas ainult veerg nimi?',
                name='query_test',
                column_name=["asukoht", "id"],
                should_exist=False,
                points=5,
            ),
            QueryStructureTest(
                title='Kas on kasutatud keelatud elemente päringus?',
                name='query_test',
                elements='limit',
                should_exist=False,
                points=20,
            ),
            QueryStructureTest(
                title='Kas on kasutatud nõutud elemente päringus?',
                name='query_test',
                elements='Maletäht',
                points=20,
            ),
            QueryDataTest(
                title='Kas on õige ridade arvuga tulemus?',
                name='query_test',
                column_name='COUNT(*)',
                expected_value=4,
                points=40,
            ),
            QueryDataTest(
                title='Kas on olemas klubi Ajurebend?',
                name='query_test',
                column_name='COUNT(*)',
                where="nimi = 'Ajurebend'",
                expected_value=1,
                points=10,
            ),
            QueryDataTest(
                title='Kas on klubi Chess 4. kohal?',
                name='query_test',
                column_name='nimi',
                where="test_id=4",
                expected_value='Chess',
                points=10,
            ),
            QueryDataTest(
                title='Kas ei ole olemas klubi Laudnikud?',
                name='query_test',
                column_name='COUNT(*)',
                where="nimi = 'Laudnikud'",
                expected_value=0,
                points=10,
            ),
        ]
    ),
]


from silmused.tests.TestDefinition import TestDefinition


class TriggerTest(TestDefinition):
    def __init__(self, name, title=None, description=None, points=0, arguments=None,
                 custom_feedback=None, action_timing=None):
        super().__init__(
            name=name,
            title=title,
            points=points,
            description=description,
            arguments=arguments,
            custom_feedback=custom_feedback,
            query=f"SELECT * FROM information_schema.views WHERE table_name = '{name}'",
        )

        self.action_timing = action_timing

    def execute(self, cursor):

        # Check that trigger name exists
        test_trigger_exists_result = self.test_trigger_exists(cursor)
        if test_trigger_exists_result is not None:
            return test_trigger_exists_result

        # Check trigger event manipulation on arguments and trigger action timing
        errors = self.test_trigger_manipulation(cursor) if len(self.arguments) > 0 else []
        if self.custom_feedback is None:
            return super().response(
                len(errors) == 0,
                {"test_type": "trigger_test",
                 "test_key": "trigger_definition_positive_feedback",
                 "params": [self.name]},
                {"test_type": "trigger_test",
                 "test_key": "trigger_definition_negative_feedback",
                 "params": [self.name, {', '.join(errors)}]},  # TODO errorid on hetkel inglise keeles
            )
        else:
            return super().response(
                False,
                {"test_type": "trigger_test",
                 "test_key": "custom_feedback",
                 "params": [self.custom_feedback]},
                {"test_type": "trigger_test",
                 "test_key": "custom_feedback",
                 "params": [self.custom_feedback]},
            )
    def test_trigger_exists(self, cursor):
        cursor.execute(f"SELECT trigger_name FROM information_schema.triggers WHERE trigger_name = '{self.name}'")

        if len(cursor.fetchall()) <= 0:
            if self.custom_feedback is None:
                return super().response(
                    False,
                    {"test_type": "trigger_test",
                     "test_key": "trigger_exists_positive_feedback",
                     "params": [self.name]},
                    {"test_type": "trigger_test",
                     "test_key": "trigger_exists_negative_feedback",
                     "params": [self.name]},
                )
            else:
                return super().response(
                    False,
                    {"test_type": "trigger_test",
                     "test_key": "custom_feedback",
                     "params": [self.custom_feedback]},
                    {"test_type": "trigger_test",
                     "test_key": "custom_feedback",
                     "params": [self.custom_feedback]},
                )
        return None

    def test_trigger_manipulation(self, cursor):
        errors = []
        for manipulation in self.arguments:
            cursor.execute(
                f"SELECT trigger_name FROM information_schema.triggers WHERE trigger_name = '{self.name}' "
                f"AND event_manipulation = '{manipulation}'")

            if len(cursor.fetchall()) <= 0:
                errors.append(f"manipulation {manipulation} was not found")

        cursor.execute(
            f"SELECT trigger_name FROM information_schema.triggers WHERE trigger_name = '{self.name}' "
            f"AND action_timing = '{self.action_timing}'")

        if len(cursor.fetchall()) <= 0:
            errors.append(f"action timing {self.action_timing} was not found")

        return errors



from silmused.ChecksLayer import ChecksLayer
from silmused.tests.QueryStructureTest import QueryStructureTest
from silmused.tests.QueryDataTest import QueryDataTest

tests = [
    ChecksLayer(
        title='Ülesanne 1 Päringu kontrollid',
        tests=[
            QueryStructureTest(
                title='Kas on olemas veerg linn?',
                name='query_test',
                column_name='linn',
                points=10,
            ),
            QueryStructureTest(
                title='Kas on olemas veerg aasta?',
                name='query_test',
                column_name='aasta',
                points=10,
            ),
            QueryStructureTest(
                title='Kas on olemas veerg pealkiri?',
                name='query_test',
                column_name='võidulaul',
                points=10,
            ),
            QueryDataTest(
                title='Kas on õige ridade arvuga tulemus?',
                name='query_test',
                column_name='COUNT(*)',
                expected_value=15,
                points=20,
            ),
            QueryDataTest(
                title='Kas on õige laul viimasel kohal?',
                name='query_test',
                column_name='võidulaul',
                where="test_id=15",
                expected_value="The Code",
                points=30,
            ),
            QueryDataTest(
                title='Kas on õige võiduaasta Heroes laulul?',
                name='query_test',
                column_name='aasta',
                where="võidulaul='Heroes'",
                expected_value="2015",
                points=30,
            ),
        ]
    ),
]


from silmused.tests.TestDefinition import TestDefinition


class ConstraintTest(TestDefinition):
    def __init__(self, name, title=None, column_name=None, constraint_name=None, constraint_type=None, description=None,
                 should_exist=True, custom_feedback=None, points=0):
        query = f"SELECT * FROM information_schema.table_constraints WHERE table_name = '{name}'"

        if constraint_name is not None:
            query += f" AND constraint_name = '{constraint_name}'"
        if constraint_type is not None:
            query += f" AND constraint_type = '{constraint_type}'"

        super().__init__(
            name=name,
            column_name=column_name,
            title=title,
            points=points,
            description=description,
            query=query,
            should_exist=should_exist,
            custom_feedback=custom_feedback
        )

        self.constraint_name = constraint_name
        self.constraint_type = constraint_type

        self.feedback = f"table {self.name}"

        if self.column_name is not None:
            self.feedback += f" with column = '{self.column_name}'"
        if self.constraint_name is not None:
            self.feedback += f" with constraint name = '{self.constraint_name}'"
        if self.constraint_type is not None:
            if self.constraint_name is None:
                self.feedback += f" with constraint type = '{self.constraint_type}'"
            else:
                self.feedback += f" and type = '{self.constraint_type}'"

    def execute(self, cursor):
        check_columns_result = self.check_columns(cursor)
        if check_columns_result is not None:
            return check_columns_result
        cursor.execute(self.query)
        result = cursor.fetchall()
        if self.should_exist:
            if self.column_name is None and self.constraint_name is None and self.constraint_type is None:
                if self.custom_feedback is None:
                    return super().response(
                        len(result) > 0,
                        {"test_type": "constraint_test",
                         "test_key": "table_constraint_should_exist_positive_feedback",
                         "params": [self.name]},
                        {"test_type": "constraint_test",
                         "test_key": "table_constraint_should_exist_negative_feedback",
                         "params": [self.name]},
                    )
                else:
                    return super().response(
                        len(result) > 0,
                        {"test_type": "constraint_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                        {"test_type": "constraint_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                    )

            elif self.column_name is not None and self.constraint_name is None and self.constraint_type is None:
                if self.custom_feedback is None:
                    return super().response(
                        len(result) > 0,
                        {"test_type": "constraint_test",
                         "test_key": "table_column_constraint_should_exist_positive_feedback",
                         "params": [self.name, self.column_name]},
                        {"test_type": "constraint_test",
                         "test_key": "table_column_constraint_should_exist_negative_feedback",
                         "params": [self.name, self.column_name]},
                    )
                else:
                    return super().response(
                        len(result) > 0,
                        {"test_type": "constraint_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                        {"test_type": "constraint_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                    )
            elif self.column_name is not None and self.constraint_name is not None and self.constraint_type is None:
                if self.custom_feedback is None:
                    return super().response(
                        len(result) > 0,
                        {"test_type": "constraint_test",
                         "test_key": "table_column_constraint_name_should_exist_positive_feedback",
                         "params": [self.name, self.column_name, self.constraint_name]},
                        {"test_type": "constraint_test",
                         "test_key": "table_column_constraint_name_should_exist_negative_feedback",
                         "params": [self.name, self.column_name, self.constraint_name]},
                    )
                else:
                    return super().response(
                        len(result) > 0,
                        {"test_type": "constraint_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                        {"test_type": "constraint_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                    )
            elif self.column_name is not None and self.constraint_type is not None:
                if self.constraint_name is None:
                    if self.custom_feedback is None:
                        return super().response(
                            len(result) > 0,
                            {"test_type": "constraint_test",
                             "test_key": "table_column_constraint_type_should_exist_positive_feedback",
                             "params": [self.name, self.column_name, self.constraint_type]},
                            {"test_type": "constraint_test",
                             "test_key": "table_column_constraint_type_should_exist_negative_feedback",
                             "params": [self.name, self.column_name, self.constraint_type]},
                        )
                    else:
                        return super().response(
                            len(result) > 0,
                            {"test_type": "constraint_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                            {"test_type": "constraint_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                        )
                else:
                    if self.custom_feedback is None:
                        return super().response(
                            len(result) > 0,
                            {"test_type": "constraint_test",
                             "test_key": "table_column_constraint_name_and_type_should_exist_positive_feedback",
                             "params": [self.name, self.column_name, self.constraint_name, self.constraint_type]},
                            {"test_type": "constraint_test",
                             "test_key": "table_column_constraint_name_and_type_should_exist_negative_feedback",
                             "params": [self.name, self.column_name, self.constraint_name, self.constraint_type]},
                        )
                    else:
                        return super().response(
                            len(result) > 0,
                            {"test_type": "constraint_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                            {"test_type": "constraint_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                        )
            else:
                if self.constraint_name is not None and self.constraint_type is None:
                    if self.custom_feedback is None:
                        return super().response(
                            len(result) > 0,
                            {"test_type": "constraint_test",
                             "test_key": "table_constraint_name_should_exist_positive_feedback",
                             "params": [self.constraint_name, self.name]},
                            {"test_type": "constraint_test",
                             "test_key": "table_constraint_name_should_exist_negative_feedback",
                             "params": [self.constraint_name, self.name]},
                        )
                    else:
                        return super().response(
                            len(result) > 0,
                            {"test_type": "constraint_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                            {"test_type": "constraint_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                        )
                elif self.constraint_name is None and self.constraint_type is not None:
                    if self.custom_feedback is None:
                        return super().response(
                            len(result) > 0,
                            {"test_type": "constraint_test",
                             "test_key": "table_constraint_type_should_exist_positive_feedback",
                             "params": [self.constraint_type, self.name]},
                            {"test_type": "constraint_test",
                             "test_key": "table_constraint_type_should_exist_negative_feedback",
                             "params": [self.constraint_type, self.name]},
                        )
                    else:
                        return super().response(
                            len(result) > 0,
                            {"test_type": "constraint_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                            {"test_type": "constraint_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                        )
                else:
                    if self.custom_feedback is None:
                        return super().response(
                            len(result) > 0,
                            {"test_type": "constraint_test",
                             "test_key": "table_constraint_name_and_type_should_exist_positive_feedback",
                             "params": [self.constraint_name, self.constraint_type, self.name]},
                            {"test_type": "constraint_test",
                             "test_key": "table_constraint_name_and_type_should_exist_negative_feedback",
                             "params": [self.constraint_name, self.constraint_type, self.name]},
                        )
                    else:
                        return super().response(
                            len(result) > 0,
                            {"test_type": "constraint_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                            {"test_type": "constraint_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                        )

        # Should exist = False
        else:
            if self.column_name is None and self.constraint_name is None and self.constraint_type is None:
                if self.custom_feedback is None:
                    return super().response(
                        len(result) == 0,
                        {"test_type": "constraint_test",
                         "test_key": "table_constraint_should_not_exist_positive_feedback",
                         "params": [self.name]},
                        {"test_type": "constraint_test",
                         "test_key": "table_constraint_should_not_exist_negative_feedback",
                         "params": [self.name]},
                    )
                else:
                    return super().response(
                        len(result) == 0,
                        {"test_type": "constraint_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                        {"test_type": "constraint_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                    )

            elif self.column_name is not None and self.constraint_name is None and self.constraint_type is None:
                if self.custom_feedback is None:
                    return super().response(
                        len(result) == 0,
                        {"test_type": "constraint_test",
                         "test_key": "table_column_constraint_should_not_exist_positive_feedback",
                         "params": [self.name, self.column_name]},
                        {"test_type": "constraint_test",
                         "test_key": "table_column_constraint_should_not_exist_negative_feedback",
                         "params": [self.name, self.column_name]},
                    )
                else:
                    return super().response(
                        len(result) == 0,
                        {"test_type": "constraint_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                        {"test_type": "constraint_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                    )
            elif self.column_name is not None and self.constraint_name is not None and self.constraint_type is None:
                if self.custom_feedback is None:
                    return super().response(
                        len(result) == 0,
                        {"test_type": "constraint_test",
                         "test_key": "table_column_constraint_name_should_not_exist_positive_feedback",
                         "params": [self.name, self.column_name, self.constraint_name]},
                        {"test_type": "constraint_test",
                         "test_key": "table_column_constraint_name_should_not_exist_negative_feedback",
                         "params": [self.name, self.column_name, self.constraint_name]},
                    )
                else:
                    return super().response(
                        len(result) == 0,
                        {"test_type": "constraint_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                        {"test_type": "constraint_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                    )
            elif self.column_name is not None and self.constraint_type is not None:
                if self.constraint_name is None:
                    if self.custom_feedback is None:
                        return super().response(
                            len(result) == 0,
                            {"test_type": "constraint_test",
                             "test_key": "table_column_constraint_type_should_not_exist_positive_feedback",
                             "params": [self.name, self.column_name, self.constraint_type]},
                            {"test_type": "constraint_test",
                             "test_key": "table_column_constraint_type_should_not_exist_negative_feedback",
                             "params": [self.name, self.column_name, self.constraint_type]},
                        )
                    else:
                        return super().response(
                            len(result) == 0,
                            {"test_type": "constraint_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                            {"test_type": "constraint_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                        )
                else:
                    if self.custom_feedback is None:
                        return super().response(
                            len(result) == 0,
                            {"test_type": "constraint_test",
                             "test_key": "table_column_constraint_name_and_type_should_not_exist_positive_feedback",
                             "params": [self.name, self.column_name, self.constraint_name, self.constraint_type]},
                            {"test_type": "constraint_test",
                             "test_key": "table_column_constraint_name_and_type_should_not_exist_negative_feedback",
                             "params": [self.name, self.column_name, self.constraint_name, self.constraint_type]},
                        )
                    else:
                        return super().response(
                            len(result) == 0,
                            {"test_type": "constraint_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                            {"test_type": "constraint_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                        )
            else:
                if self.constraint_name is not None and self.constraint_type is None:
                    if self.custom_feedback is None:
                        return super().response(
                            len(result) == 0,
                            {"test_type": "constraint_test",
                             "test_key": "table_constraint_name_should_not_exist_positive_feedback",
                             "params": [self.constraint_name, self.name]},
                            {"test_type": "constraint_test",
                             "test_key": "table_constraint_name_should_not_exist_negative_feedback",
                             "params": [self.constraint_name, self.name]},
                        )
                    else:
                        return super().response(
                            len(result) == 0,
                            {"test_type": "constraint_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                            {"test_type": "constraint_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                        )
                elif self.constraint_name is None and self.constraint_type is not None:
                    if self.custom_feedback is None:
                        return super().response(
                            len(result) == 0,
                            {"test_type": "constraint_test",
                             "test_key": "table_constraint_type_should_not_exist_positive_feedback",
                             "params": [self.constraint_type, self.name]},
                            {"test_type": "constraint_test",
                             "test_key": "table_constraint_type_should_not_exist_negative_feedback",
                             "params": [self.constraint_type, self.name]},
                        )
                    else:
                        return super().response(
                            len(result) == 0,
                            {"test_type": "constraint_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                            {"test_type": "constraint_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                        )
                else:
                    if self.custom_feedback is None:
                        return super().response(
                            len(result) == 0,
                            {"test_type": "constraint_test",
                             "test_key": "table_constraint_name_and_type_should_not_exist_positive_feedback",
                             "params": [self.constraint_name, self.constraint_type, self.name]},
                            {"test_type": "constraint_test",
                             "test_key": "table_constraint_name_and_type_should_not_exist_negative_feedback",
                             "params": [self.constraint_name, self.constraint_type, self.name]},
                        )
                    else:
                        return super().response(
                            len(result) == 0,
                            {"test_type": "constraint_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                            {"test_type": "constraint_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                        )

    def check_columns(self, cursor):
        if self.column_name is None or self.constraint_type == 'CHECK':
            return None

        query = f"SELECT * FROM information_schema.key_column_usage WHERE table_name = '{self.name}'"

        if self.constraint_name is not None:
            query += f" AND constraint_name = '{self.constraint_name}'"

        cursor.execute(query)
        result = cursor.fetchall()

        columns = [res[6] for res in result]

        #print(self.column_name, columns)

        if type(self.column_name) == str and self.column_name in columns:
            return None
        elif type(self.column_name) == list:
            is_correct = True
            for column_name in self.column_name:
                if not column_name in columns:
                    is_correct = False
                    break
            if is_correct:
                return None
        if self.constraint_name is None:
            if self.custom_feedback is None:
                return super().response(
                    False,
                    None,
                    {"test_type": "constraint_test",
                     "test_key": "multi_column_name_negative_feedback",
                     "params": [self.column_name, columns]},
                )
            else:
                return super().response(
                    False,
                    {"test_type": "constraint_test",
                     "test_key": "custom_feedback",
                     "params": [self.custom_feedback]},
                    {"test_type": "constraint_test",
                     "test_key": "custom_feedback",
                     "params": [self.custom_feedback]},
                )
        else:
            if len(columns) > 0:
                if self.custom_feedback is None:
                    return super().response(
                        False,
                        None,
                        {"test_type": "constraint_test",
                         "test_key": "multi_column_constraint_name_negative_feedback",
                         "params": [self.constraint_name, self.column_name, columns]},
                    )
                else:
                    return super().response(
                        False,
                        {"test_type": "constraint_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                        {"test_type": "constraint_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                    )
            else:
                if self.custom_feedback is None:
                    return super().response(
                        False,
                        None,
                        {"test_type": "constraint_test",
                         "test_key": "multi_column_constraint_name_column_not_found_negative_feedback",
                         "params": [self.constraint_name, self.column_name]},
                    )
                else:
                    return super().response(
                        False,
                        {"test_type": "constraint_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                        {"test_type": "constraint_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                    )



from silmused.ExecuteLayer import ExecuteLayer
from silmused.TitleLayer import TitleLayer
from .tests.DataTest import DataTest
from .tests.StructureTest import StructureTest
from .tests.ConstraintTest import ConstraintTest
from .tests.FunctionTest import FunctionTest
from .tests.IndexTest import IndexTest
from .tests.ProcedureTest import ProcedureTest
from .tests.TriggerTest import TriggerTest
from .tests.ViewTest import ViewTest
import silmused.Runner

__all__ = [
    'ExecuteLayer',
    'TitleLayer',
    'DataTest',
    'StructureTest',
    'ConstraintTest',
    'FunctionTest',
    'IndexTest',
    'ProcedureTest',
    'TriggerTest',
    'ViewTest',
    'Runner',
]


from silmused.ChecksLayer import ChecksLayer
from silmused.tests.QueryStructureTest import QueryStructureTest
from silmused.tests.QueryDataTest import QueryDataTest

tests = [
    ChecksLayer(
        title='Mitte Agregeeriva Päringu kontrollid',
        tests=[
            QueryStructureTest(
                title='Kas on olemas veerg pealkiri?',
                name='query_test',
                column_name='pealkiri',
                points=25,
            ),
            QueryDataTest(
                title='Kas on õige ridade arvuga tulemus?',
                name='query_test',
                column_name='COUNT(*)',
                expected_value=5,
                points=25,
            ),
            QueryDataTest(
                title='Kas leidub laul "Nova Deca"?',
                name='query_test',
                column_name='pealkiri',
                where="pealkiri = 'Nova deca'",
                points=25,
            ),
            QueryDataTest(
                title='Kas leidub laul "Eaea"?',
                name='query_test',
                column_name='pealkiri',
                where="pealkiri = 'Eaea'",
                points=25,
            ),
        ]
    ),
]


from silmused.ChecksLayer import ChecksLayer
from silmused.tests.QueryStructureTest import QueryStructureTest
from silmused.tests.QueryDataTest import QueryDataTest
tests = [
    ChecksLayer(
        title='Ülesanne 1 Päringu kontrollid',
        tests=[
            QueryStructureTest(
                title='Kas on olemas veerg eesnimi?',
                name='query_test',
                column_name='eesnimi',
                points=15,
            ),
            QueryStructureTest(
                title='Kas on olemas veerg perenimi?',
                name='query_test',
                column_name='perenimi',
                points=15,
            ),
            QueryStructureTest(
                title='Kas tulemuses on ainult veerud eesnimi ja perenimi?',
                name='query_test',
                column_name=['id','isikukood','klubi','synniaeg','sugu','ranking'],
                should_exist=False,
                points=10,
            ),
            QueryStructureTest(
                title='Kas lahenduses on kasutatud LIMITit?',
                name='query_test',
                elements=['LIMIT'],
                should_exist=False,
                points=20,
            ),
            QueryDataTest(
                title='Kas on õige ridade arvuga tulemus?',
                name='query_test',
                column_name='COUNT(*)',
                expected_value=5,
                points=20,
            ),
            QueryDataTest(
                title='Kas on õige isik kõige noorem?',
                name='query_test',
                column_name='eesnimi',
                where="test_id=1",
                expected_value="Sander",
                points=20,
            ),
            QueryDataTest(
                title='Kas viies noorim on õige isik?',
                name='query_test',
                column_name='eesnimi',
                where="test_id=5",
                expected_value="Keiu",
                points=20,
            ),
        ]
    ),
]


from silmused.ChecksLayer import ChecksLayer
from silmused.tests.QueryStructureTest import QueryStructureTest
from silmused.tests.QueryDataTest import QueryDataTest

tests = [
    ChecksLayer(
        title='Ülesanne 2 Päringu kontrollid',
        tests=[
            QueryStructureTest(
                title='Kas on olemas veerg id?',
                name='query_test',
                column_name='id',
                points=10,
            ),
            QueryStructureTest(
                title='Kas on olemas veerg linn?',
                name='query_test',
                column_name='linn',
                points=10,
            ),
            QueryStructureTest(
                title='Kas on olemas veerg finaali_kuupaev?',
                name='query_test',
                column_name='finaali_kuupaev',
                points=10,
            ),
            QueryStructureTest(
                title='Kas on olemas veerg riik_id?',
                name='query_test',
                column_name='riik_id',
                points=10,
            ),
            QueryDataTest(
                title='Kas on õige ridade arvuga tulemus?',
                name='query_test',
                column_name='COUNT(*)',
                expected_value=5,
                points=30,
            ),
            QueryDataTest(
                title='Kas kontroll linn on olemas?',
                name='query_test',
                column_name='linn',
                where="id=3",
                expected_value="Stockholm",
                points=30,
            ),
        ]
    ),
]


from silmused.ChecksLayer import ChecksLayer
from silmused.tests.QueryStructureTest import QueryStructureTest
from silmused.tests.QueryDataTest import QueryDataTest

tests = [
    ChecksLayer(
        title='Ülesanne 1 Päringu kontrollid',
        tests=[
            QueryStructureTest(
                title='Kas on olemas veerg riik?',
                name='query_test',
                column_name='riik',
                points=10,
            ),
            QueryStructureTest(
                title='Kas on olemas veerg laulude arv?',
                name='query_test',
                column_name='laulude arv',
                points=10,
            ),
            QueryStructureTest(
                title='Kas on olemas veerg keskmine punktisumma?',
                name='query_test',
                column_name='keskmine punktisumma',
                points=10,
            ),
            QueryDataTest(
                title='Kas on õige ridade arvuga tulemus?',
                name='query_test',
                column_name='COUNT(*)',
                expected_value=4,
                points=20,
            ),
            QueryDataTest(
                title='Kas on õige arv laule Eestil jõudnud finaali?',
                name='query_test',
                column_name='"laulude arv"',
                where="riik='Estonia'",
                expected_value=10,
                points=30,
            ),
            QueryDataTest(
                title='Kas on õige keskmine punktisumma Soomel?',
                name='query_test',
                column_name='ROUND("keskmine punktisumma",1)',
                where="riik='Finland'",
                expected_value=123.7,
                points=20,
            ),
        ]
    ),
]


from silmused.tests.TestDefinition import TestDefinition


class QueryDataTest(TestDefinition):
    def __init__(self, name, title=None, column_name=None, should_exist=True, where=None, join=None, description=None,
                 expected_value=None, expected_value_query=None, column_name_fallback=None, custom_feedback=None,
                 points=0):

        if column_name is not None and not isinstance(column_name, str):
            raise Exception('Parameter "column_name" must be a string')
        if expected_value_query is not None and not isinstance(expected_value_query, str):
            raise Exception('Parameter "expected_value_query" must be a string')
        if column_name is not None:
            self.is_count = False if column_name.lower().find("count") == -1 else True
        if column_name_fallback is not None and not isinstance(column_name, list):
            raise Exception('Parameter "column_name_fallback" must be a list')
        if isinstance(expected_value, list):
            self.expected_value_list = True
            if isinstance(expected_value[0], str):
                self.expected_value_group = "strings"
            else:
                self.expected_value_group = "numbers"
                min_value = None
                max_value = None
                for value in expected_value:
                    if isinstance(value, str):
                        raise Exception('Ranged expected value cannot be a string')
                    if min_value is None:
                        min_value = value
                    elif value < min_value:
                        min_value = value
                    if max_value is None:
                        max_value = value
                    elif value > max_value:
                        max_value = value
                self.expected_min_value = min_value
                self.expected_max_value = max_value
        else:
            self.expected_value_list = False

        super().__init__(
            name=name,
            title=title,
            where=where,
            join=join,
            points=points,
            description=description,
            query=f"SELECT {column_name if column_name is not None else '*'} FROM {name}",
            should_exist=should_exist,
            expected_value=expected_value,
            custom_feedback=custom_feedback,
            column_name_fallback=column_name_fallback,
            expected_value_query=expected_value_query,
        )

        self.column_name = column_name
        self.where = where
        self.join = join

    def execute(self, cursor):
        if self.expected_value_query is not None:
            cursor.execute(self.expected_value_query)
            result = cursor.fetchall()
            self.expected_value = result[0][0]
        if self.column_name_fallback is not None:
            self.column_name = self.check_alternative_columns(cursor)
            self.query = (f"SELECT {self.column_name if self.column_name is not None else '*'} FROM {self.name}" +
                          f" WHERE ({self.where})") if self.where is not None else ""

        cursor.execute(self.query)
        result = cursor.fetchall()

        # TODO Add an overall test response to the end, incase of oversight
        if self.expected_value is None:
            if self.should_exist:
                if self.column_name is None:
                    if self.custom_feedback is None:
                        return super().response(
                            len(result) > 0,
                            {"test_type": "query_data_test",
                             "test_key": "query_not_expected_value_should_exist_positive_feedback"},
                            {"test_type": "query_data_test",
                             "test_key": "query_not_expected_value_should_exist_negative_feedback"},

                        )
                    else:
                        return super().response(
                            len(result) > 0,
                            {"test_type": "query_data_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                            {"test_type": "query_data_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                        )
                else:
                    if self.is_count:
                        if self.custom_feedback is None:
                            return super().response(
                                result[0][0] > 0,
                                {"test_type": "query_data_test",
                                 "test_key": "query_column_not_expected_value_should_exist_positive_feedback",
                                 "params": [self.column_name]},
                                {"test_type": "query_data_test",
                                 "test_key": "query_column_not_expected_value_should_exist_negative_feedback",
                                 "params": [self.column_name]},

                            )
                        else:
                            return super().response(
                                result[0][0] > 0,
                                {"test_type": "query_data_test",
                                 "test_key": "custom_feedback",
                                 "params": [self.custom_feedback]},
                                {"test_type": "query_data_test",
                                 "test_key": "custom_feedback",
                                 "params": [self.custom_feedback]},
                            )
                    # Same feedback test_keys
                    else:
                        if self.custom_feedback is None:
                            # TODO Should check if all results are not None
                            return super().response(
                                len(result) > 0 and result[0][0] is not None,
                                {"test_type": "query_data_test",
                                 "test_key": "query_column_not_expected_value_should_exist_positive_feedback",
                                 "params": [self.column_name]},
                                {"test_type": "query_data_test",
                                 "test_key": "query_column_not_expected_value_should_exist_negative_feedback",
                                 "params": [self.column_name]},
                            )
                        else:
                            return super().response(
                                len(result) > 0 and result[0][0] is not None,
                                {"test_type": "query_data_test",
                                 "test_key": "custom_feedback",
                                 "params": [self.custom_feedback]},
                                {"test_type": "query_data_test",
                                 "test_key": "custom_feedback",
                                 "params": [self.custom_feedback]},
                            )
            # should_exist = False
            else:
                if self.column_name is None:
                    if self.custom_feedback is None:
                        return super().response(
                            len(result) == 0,
                            {"test_type": "query_data_test",
                             "test_key": "query_not_expected_value_should_not_exist_positive_feedback"},
                            {"test_type": "query_data_test",
                             "test_key": "query_not_expected_value_should_not_exist_negative_feedback"},

                        )
                    else:
                        return super().response(
                            len(result) == 0,
                            {"test_type": "query_data_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                            {"test_type": "query_data_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                        )
                else:
                    if self.custom_feedback is None:
                        return super().response(
                            len(result) == 0,
                            {"test_type": "query_data_test",
                             "test_key": "query_column_not_expected_value_should_not_exist_positive_feedback",
                             "params": [self.column_name]},
                            {"test_type": "query_data_test",
                             "test_key": "query_column_not_expected_value_should_not_exist_negative_feedback",
                             "params": [self.column_name]},

                        )
                    else:
                        return super().response(
                            len(result) == 0,
                            {"test_type": "query_data_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                            {"test_type": "query_data_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                        )
        # expected value is not None
        else:
            if self.should_exist:
                if len(result) == 0:
                    if self.custom_feedback is None:
                        return super().response(
                            False,
                            "",
                            {"test_type": "query_data_test",
                             "test_key": "query_no_result",
                             "params": []},
                        )
                    else:
                        return super().response(
                            False,
                            {"test_type": "query_data_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                            {"test_type": "query_data_test",
                             "test_key": "custom_feedback",
                             "params": [self.custom_feedback]},
                        )
                if self.expected_value == 'NULL' or self.expected_value == 'None':
                    # TODO Should check if all results are None
                    if result[0][0] is None and len(result) > 0:
                        if self.custom_feedback is None:
                            return super().response(
                                result[0][0] is None,
                                {"test_type": "query_data_test",
                                 "test_key": "query_expected_value_should_exist_positive_feedback",
                                 "params": [self.expected_value, self.name, self.column_name]},
                                {"test_type": "query_data_test",
                                 "test_key": "query_expected_value_should_exist_negative_feedback",
                                 "params": ['NULL', str(result[0][0]), self.name, self.column_name]},
                            )
                        else:
                            return super().response(
                                result[0][0] is None,
                                {"test_type": "query_data_test",
                                 "test_key": "custom_feedback",
                                 "params": [self.custom_feedback]},
                                {"test_type": "query_data_test",
                                 "test_key": "custom_feedback",
                                 "params": [self.custom_feedback]},
                            )
                    else:
                        if self.custom_feedback is None:
                            return super().response(
                                False,
                                '',
                                {"test_type": "query_data_test",
                                 "test_key": "query_expected_value_should_exist_no_result_negative_feedback",
                                 "params": [self.column_name]},
                            )
                        else:
                            return super().response(
                                self.expected_min_value <= result[0][0] <= self.expected_max_value,
                                {"test_type": "query_data_test",
                                 "test_key": "custom_feedback",
                                 "params": [self.custom_feedback]},
                                {"test_type": "query_data_test",
                                 "test_key": "custom_feedback",
                                 "params": [self.custom_feedback]},
                            )
                elif self.expected_value_list:
                    if self.expected_value_group == "numbers":
                        if len(result) > 0:
                            if self.custom_feedback is None:
                                return super().response(
                                    self.expected_min_value <= result[0][0] <= self.expected_max_value,
                                    {"test_type": "query_data_test",
                                     "test_key": "query_expected_value_group_numbers_positive_feedback",
                                     "params": [str(result[0][0]), self.expected_min_value, self.expected_max_value,
                                                self.column_name]},
                                    {"test_type": "query_data_test",
                                     "test_key": "query_expected_value_group_numbers_negative_feedback",
                                     "params": [str(result[0][0]), self.expected_min_value, self.expected_max_value,
                                                self.column_name]},
                                )
                            else:
                                return super().response(
                                    self.expected_min_value <= result[0][0] <= self.expected_max_value,
                                    {"test_type": "query_data_test",
                                     "test_key": "custom_feedback",
                                     "params": [self.custom_feedback]},
                                    {"test_type": "query_data_test",
                                     "test_key": "custom_feedback",
                                     "params": [self.custom_feedback]},
                                )
                        else:
                            if self.custom_feedback is None:
                                return super().response(
                                    False,
                                    '',
                                    {"test_type": "query_data_test",
                                     "test_key": "query_expected_value_group_numbers_no_result_negative_feedback",
                                     "params": [self.column_name]},
                                )
                            else:
                                return super().response(
                                    self.expected_min_value <= result[0][0] <= self.expected_max_value,
                                    {"test_type": "query_data_test",
                                     "test_key": "custom_feedback",
                                     "params": [self.custom_feedback]},
                                    {"test_type": "query_data_test",
                                     "test_key": "custom_feedback",
                                     "params": [self.custom_feedback]},
                                )
                    elif self.expected_value_group == "strings":
                        if len(result) > 0:
                            if self.custom_feedback is None:
                                return super().response(
                                    result[0][0] in self.expected_value,
                                    {"test_type": "query_data_test",
                                     "test_key": "query_expected_value_group_strings_positive_feedback",
                                     "params": [str(result[0][0]), self.expected_value, self.column_name]},
                                    {"test_type": "query_data_test",
                                     "test_key": "query_expected_value_group_strings_negative_feedback",
                                     "params": [str(result[0][0]), self.expected_value, self.column_name]},
                                )
                            else:
                                return super().response(
                                    result[0][0] in self.expected_value,
                                    {"test_type": "query_data_test",
                                     "test_key": "custom_feedback",
                                     "params": [self.custom_feedback]},
                                    {"test_type": "query_data_test",
                                     "test_key": "custom_feedback",
                                     "params": [self.custom_feedback]},
                                )
                        else:
                            if self.custom_feedback is None:
                                return super().response(
                                    False,
                                    '',
                                    {"test_type": "query_data_test",
                                     "test_key": "query_expected_value_group_strings_no_result_negative_feedback",
                                     "params": [self.column_name]},
                                )
                            else:
                                return super().response(
                                    self.expected_min_value <= result[0][0] <= self.expected_max_value,
                                    {"test_type": "query_data_test",
                                     "test_key": "custom_feedback",
                                     "params": [self.custom_feedback]},
                                    {"test_type": "query_data_test",
                                     "test_key": "custom_feedback",
                                     "params": [self.custom_feedback]},
                                )
                else:
                    if self.custom_feedback is None:
                        if not isinstance(result[0][0], str) and not isinstance(self.expected_value, str):
                            return super().response(
                                result[0][0] == self.expected_value,
                                {"test_type": "query_data_test",
                                 "test_key": "query_expected_value_should_exist_positive_feedback",
                                 "params": [self.expected_value, self.column_name]},
                                {"test_type": "query_data_test",
                                 "test_key": "query_expected_value_should_exist_negative_feedback",
                                 "params": [self.expected_value, str(result[0][0]), self.column_name]},
                            )
                        else:
                            return super().response(
                                str(result[0][0]) == str(self.expected_value),
                                {"test_type": "query_data_test",
                                 "test_key": "query_expected_value_should_exist_positive_feedback",
                                 "params": [self.expected_value, self.column_name]},
                                {"test_type": "query_data_test",
                                 "test_key": "query_expected_value_should_exist_negative_feedback",
                                 "params": [self.expected_value, str(result[0][0]), self.column_name]},
                            )
                    else:
                        if not isinstance(result[0][0], str) and not isinstance(self.expected_value, str):
                            return super().response(
                                result[0][0] == self.expected_value,
                                {"test_type": "query_data_test",
                                 "test_key": "custom_feedback",
                                 "params": [self.custom_feedback]},
                                {"test_type": "query_data_test",
                                 "test_key": "custom_feedback",
                                 "params": [self.custom_feedback]},
                            )
                        else:
                            return super().response(
                                str(result[0][0]) == str(self.expected_value),
                                {"test_type": "query_data_test",
                                 "test_key": "custom_feedback",
                                 "params": [self.custom_feedback]},
                                {"test_type": "query_data_test",
                                 "test_key": "custom_feedback",
                                 "params": [self.custom_feedback]},
                            )
            # expected value shouldn't exist
            else:
                if self.custom_feedback is None:
                    return super().response(
                        str(result[0][0]) != str(self.expected_value),
                        {"test_type": "query_data_test",
                         "test_key": "query_expected_value_should_not_exist_positive_feedback",
                         "params": [self.expected_value, str(result[0][0]), self.column_name]},
                        {"test_type": "query_data_test",
                         "test_key": "query_expected_value_should_not_exist_negative_feedback",
                         "params": [self.expected_value, str(result[0][0]), self.column_name]},
                    )
                else:
                    return super().response(
                        str(result[0][0]) != str(self.expected_value),
                        {"test_type": "query_data_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                        {"test_type": "query_data_test",
                         "test_key": "custom_feedback",
                         "params": [self.custom_feedback]},
                    )
        return super().response(
            str(result[0][0]) != str(self.expected_value),
            {"test_type": "query_data_test",
             "test_key": "no_feedback"},
            {"test_type": "query_data_test",
             "test_key": "no_feedback"},
        )

    def check_alternative_columns(self, cursor):
        for c_name in self.column_name_fallback:
            query = (f"SELECT column_name FROM information_schema.columns WHERE table_name = '{self.name}' "
                     f"AND column_name ILIKE '{c_name}'")
            cursor.execute(query)
            result = cursor.fetchall()
            if len(result[0][0]) > 0:
                return result[0][0]
        return self.column_name



from silmused import ViewTest
from silmused.ChecksLayer import ChecksLayer
from silmused.tests.DataTest import DataTest


"""
tests = [
    # Kodutöö 4 ülesanne 4
    ChecksLayer(
        title='Tabeli Isikud kontrollid',
        tests=[
            StructureTest(
                title='Kas tabeli nimi on muudetud nimeks Isikud?',
                name='isikud',
                points=25,
            ),
            StructureTest(
                title='Kas on loodud veerg elukoht?',
                name='isikud',
                column_name='elukoht',
                points=25,
            ),
            StructureTest(
                title='Kas on lisatud id numbriga veerg?',
                name='isikud',
                arguments=['COUNT(*)'],
                expected_value=9,
                points=25,
            ),
            DataTest(
                title='Kas tabelis isikud on andmed olemas?',
                name='isikud',
                points=25,
            ),
        ]
    ),
]"""
"""
tests = [
    # Praktikum 6 ül 1
    ChecksLayer(
        title='Vaate v_voidulaulud kontrollid',
        tests=[
            ViewTest(
                title='Kas on olemas vaade v_voidulaulud?',
                name='v_voidulaulud',
                points=10,
            ),
            ViewTest(
                title='Kas on olemas veerg kuupäev?',
                name='v_voidulaulud',
                column_name='kuupäev',
                points=10,
            ),
            ViewTest(
                title='Kas on olemas veerg riik?',
                name='v_voidulaulud',
                column_name='riik',
                points=10,
            ),
            ViewTest(
                title='Kas on olemas veerg laul?',
                name='v_voidulaulud',
                column_name='laul',
                points=10,
            ),
            DataTest(
                title='Kas on õige riik laulul TOY?',
                name='v_voidulaulud',
                column_name='riik',
                where="laul='TOY'",
                expected_value='Israel',
                isView=True,
                points=10,
            ),
            DataTest(
                title='Kas on õige finaali kuupäev laulul Heroes?',
                name='v_voidulaulud',
                column_name='kuupäev',
                where="laul='Heroes'",
                expected_value='2015-05-23',
                isView=True,
                points=10,
            )
        ]
    ),
]"""

tests = [
    ChecksLayer(
        title='Vaate v_ansamblid kontrollid',
        tests=[
            ViewTest(
                title='Kas om olemas vaade v_ansamblid?',
                name='v_ansamblid',
                points=10,
            ),
            ViewTest(
                title='Kas om olemas veerg linn?',
                name='v_ansamblid',
                column_name='linn',
                points=10,
            ),
            ViewTest(
                title='Kas om olemas veerg aasta?',
                name='v_ansamblid',
                column_name='aasta',
                points=10,
            ),
            ViewTest(
                title='Kas om olemas veerg pealkiri?',
                name='v_ansamblid',
                column_name='pealkiri',
                points=10,
            ),
            ViewTest(
                title='Kas om olemas veerg lavanimi?',
                name='v_ansamblid',
                column_name='lavanimi',
                points=10,
            ),
            DataTest(
                title='Kas on õige tulemuste arv?',
                name='v_ansamblid',
                column_name='COUNT(*)',
                expected_value=148,
                isView=True,
                points=25,
            ),
            DataTest(
                title='Kas on õige finaali kuupäev',
                name='v_ansamblid',
                column_name='aasta',
                where="lavanimi='Sinplus'",
                expected_value=2012,
                isView=True,
                points=25,
            )
        ]
    ),
    ChecksLayer(
        title='Vaate mv_samataht kontrollid',
        tests=[
            ViewTest(
                title='Kas om olemas vaade mv_samataht?',
                name='mv_samataht',
                isMaterialized=True,
                points=25,
            ),
            DataTest(
                title='Kas on õige tulemuste arv?',
                name='mv_samataht',
                column_name='COUNT(*)',
                expected_value=10,
                isView=True,
                points=25,
            ),
            DataTest(
                title='Kas on õige finaali kuupäev',
                name='mv_samataht',
                column_name='COUNT(*)',
                where="left(pealkiri,1)!=left(lavanimi,1)",
                expected_value=0,
                isView=True,
                points=50,
            )
        ]
    ),
    ChecksLayer(
        title='Vaate v_synniriigid kontrollid',
        tests=[
            ViewTest(
                title='Kas om olemas vaade v_synniriigid?',
                name='v_synniriigid',
                points=25,
            ),
            ViewTest(
                title='Kas om olemas veerg riik?',
                name='v_synniriigid',
                column_name='riik',
                points=10,
            ),
            ViewTest(
                title='Kas om olemas veerg lauljate arv?',
                name='v_synniriigid',
                column_name='lauljate arv',
                points=10,
            ),
            DataTest(
                title='Kas on õige tulemuste arv?',
                name='v_synniriigid',
                column_name='COUNT(*)',
                expected_value=64,
                isView=True,
                points=25,
            ),
            DataTest(
                title='Kas on õige suurim lauljate arv?',
                name='v_synniriigid',
                column_name='MAX("lauljate arv")',
                expected_value=17,
                isView=True,
                points=30,
            )
        ]
    ),
    ChecksLayer(
        title='Vaate v_kestus kontrollid',
        tests=[
            ViewTest(
                title='Kas om olemas vaade v_kestus?',
                name='v_kestus',
                points=20,
            ),
            ViewTest(
                title='Kas om olemas veerg kestus?',
                name='v_kestus',
                column_name='kestus',
                points=10,
            ),
            ViewTest(
                title='Kas om olemas veerg keskmised punktid?',
                name='v_kestus',
                column_name='keskmised punktid',
                points=10,
            ),
            ViewTest(
                title='Kas om olemas veerg koht finaalis?',
                name='v_kestus',
                column_name='koht finaalis',
                points=10,
            ),
            DataTest(
                title='Kas on õige tulemuste arv?',
                name='v_kestus',
                column_name='COUNT(*)',
                expected_value=59,
                isView=True,
                points=10,
            ),
            DataTest(
                title='Kas on õige kestus, kui keskmine koht on 1?',
                name='v_kestus',
                column_name='kestus',
                where='"koht finaalis"=1',
                expected_value='00:03:11',
                isView=True,
                points=20,
            ),
            DataTest(
                title='Kas on õigel kohal kõige kõrgemad keskmised punktid?',
                name='v_kestus',
                column_name='round("koht finaalis",0)',
                where='"keskmised punktid"=466',
                expected_value='2',
                isView=True,
                points=20,
            )
        ]
    ),
]


from silmused.TitleLayer import TitleLayer
from silmused.ChecksLayer import ChecksLayer
from silmused.ExecuteLayer import ExecuteLayer
from silmused.tests.DataTest import DataTest
from silmused.tests.StructureTest import StructureTest
from silmused.tests.ConstraintTest import ConstraintTest
from silmused.tests.FunctionTest import FunctionTest
from silmused.tests.IndexTest import IndexTest
from silmused.tests.ProcedureTest import ProcedureTest
from silmused.tests.TriggerTest import TriggerTest
from silmused.tests.ViewTest import ViewTest

tests = [
    # Kodutöö 4 kontrollid
    ChecksLayer(
        title='Vaate v_keskminepartii kontrollid',
        tests=[
            DataTest(
                title='Kas turniiril Plekkkarikas 2010 keskmine partii arv on õige?',
                name='v_keskminepartii',
                column_name='ROUND(keskmine_partii, 3)',
                where="turniiri_nimi = 'Plekkkarikas 2010'",
                expected_value=[23.764,24.27],
                isView=True,
                points=1,
            ),
            DataTest(
                title='Kas turniiril Kolme klubi kohtumine keskmine partii arv on õige?',
                name='v_keskminepartii',
                column_name='ROUND(keskmine_partii, 2)',
                where="turniiri_nimi = 'Kolme klubi kohtumine'",
                expected_value=[23.03,23.7],
                isView=True,
                points=1,
            ),
        ]
    ),
]


from silmused.ChecksLayer import ChecksLayer
from silmused.tests.QueryStructureTest import QueryStructureTest
from silmused.tests.QueryDataTest import QueryDataTest
tests = [
    ChecksLayer(
        title='Ülesanne 2 Päringu kontrollid',
        tests=[
            QueryStructureTest(
                title='Kas on olemas veerg eesnimi?',
                name='query_test',
                column_name='eesnimi',
                points=10,
            ),
            QueryStructureTest(
                title='Kas on olemas veerg perenimi?',
                name='query_test',
                column_name='perenimi',
                points=10,
            ),
            QueryStructureTest(
                title='Kas on olemas veerg sugu?',
                name='query_test',
                column_name='sugu',
                points=10,
            ),
            QueryStructureTest(
                title='Kas on olemas veerg vanus?',
                name='query_test',
                column_name='vanus',
                points=10,
            ),
            QueryStructureTest(
                title='Kas tulemuses on ainult nõutud veerud?',
                name='query_test',
                column_name=['id','isikukood','klubi','synniaeg','ranking'],
                should_exist=False,
                points=5,
            ),
            QueryStructureTest(
                title='Kas päringus on kasutatud nõutud tingimusi?',
                name='query_test',
                elements='ets',
                points=10,
            ),
            QueryDataTest(
                title='Kas on õige ridade arvuga tulemus?',
                name='query_test',
                column_name='COUNT(*)',
                expected_value=4,
                points=25,
            ),
            QueryDataTest(
                title='Kas on õige vanus Arvo Metsal?',
                name='query_test',
                column_name='vanus',
                where="eesnimi = 'Arvo' and perenimi = 'Mets'",
                expected_value=86,
                points=20,
            ),
        ]
    ),
]


import sys


class ExecuteLayer:
    def __init__(self, query):
        self.query = query

    def run(self, cursor):
        try:
            cursor.execute(self.query)

            return {
                'type': 'execution',
                'message': 'Success',
                'query': self.query,
            }
        except:
            cursor.execute('ROLLBACK')

            return {
                'type': 'execution',
                'message': 'Failure',
                'error': sys.exc_info(),
                'query': self.query,
            }